#!/usr/bin/perl
#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       config.cgi                                           #
#  Description: a perl script to read/write data from a ajax call    #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2014-2016 by QTip                                    #
#  License:     GNU GPLv3 (see LICENSE)                              #
#  ----------------------------------------------------------------  #
#  Version:     0.6 - 09/01/2016                                     #
#  for more information check the changelog                          #
#********************************************************************#

$user = `/usr/syno/synoman/webman/modules/authenticate.cgi`;
$user =~ s/^\s+|\s+$//g;
exit if ($user eq '');

use strict;
use warnings;

use FindBin qw($Bin);
use File::Spec;
use lib ("$Bin/../../lib");

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use JSON::XS;
use Fcntl qw(:flock);
#use Data::Dumper;

## initialize vars #################
my (@parts,$part,$buffer,$value,$name,%param,$raw_data,$options,$status_data,$status_new,@grep_ret);
my @path = File::Spec->splitdir($Bin);
my $packagehome = "/var/packages/$path[3]";
my $configpath = "$packagehome/etc/";
my $status_file = "$configpath/status.json";
my $pidfile = "/var/run/ddud.pid";
my $debug_file = "$packagehome/target/log/ddud_debug.log";

use DDNSUpdater::CryptDecrypt;

sub get_pid {
     my $pidfile = shift;

     if (-e $pidfile) {
          if (!open(FD,"<",$pidfile)) {
               return 0;
        	} else {
               my $PID = <FD>;
               close(FD);
               return $PID;
          }
     }
     return 0;
}
###################################

print "Content-type: application/json; charset=UTF-8\n\n";

if ($ENV{'REQUEST_METHOD'} eq "POST") {
     read(STDIN,$buffer,$ENV{"CONTENT_LENGTH"});
     @parts = split(/\&/,$buffer);
     foreach $part (@parts) {
          ($name,$value) = split(/=/,$part);
          $value =~ tr/+/ /;
          $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
          # prevent include of server side includes
          $value =~ s/<!--(.|\n)*-->//g;
          $param{$name} = $value;
     }
     if ($param{'action'} eq "read" && $param{'file'} ne "") {
          my $config_file = $configpath.$param{'file'}.".json";
          if (-e $config_file) {
               if (open(DAT,"<",$config_file)) {
                    flock(DAT, LOCK_EX);
                    $raw_data = <DAT>;
                    close(DAT);
               } else {
                    close(DAT);
                    print '{"success":false}';
                    exit;
               }
          } else {
               print '{"'.$param{'file'}.'":[]}';
               exit;
          }
          if ($param{'file'} eq "config") {
               if (-e $status_file) {
                    if (open(DAT,"<",$status_file)) {
                         flock(DAT, LOCK_EX);
                         $status_data = <DAT>;
                         close(DAT);
                    } else {
                         close(DAT);
                         exit;
                    }
               }
               if ($raw_data) {
                    my $config = JSON::XS->new->decode($raw_data);
                    my $status = JSON::XS->new->decode($status_data) if ($status_data);
                    my $st = ();
                    foreach my $cf (@{$config->{'config'}}) {
                         $cf->{'password'} = '0:1:2:3:4:5:6:7' if ($cf->{'password'} ne "");
                         if ($status) {
                              $st = $status->{$cf->{'id'}};
                              next if (!defined $st);
#                              $cf->{'ip'} = $st->{'ip'} if (($cf->{'iptype'} eq "auto" || $cf->{'iptype'} ne "auto" && $cf->{'offline'}) && $st->{'ip'} ne "");
                              $cf->{'ip'} = $st->{'ip'} if (($cf->{'iptype'} eq "auto" || $cf->{'iptype'} eq "syncdom4" || $cf->{'iptype'} eq "syncdom6") && $st->{'ip'} ne "");
                              $cf->{'wtime'} = $st->{'wtime'};
                              $cf->{'ltime'} = $st->{'ltime'};
                              $cf->{'status'} = $st->{'status'};
                         }
                    }
                    $raw_data = JSON::XS->new->encode($config);
#                    $raw_data =~ s/"id":"([0-9]+)"/"id":$1/g;
               }
          }
          print $raw_data;
     } elsif ($param{'action'} eq "write" && $param{'file'} ne "" && $param{'value'} ne "") {
          if ($param{'file'} eq "config") {
               my $config_file = $configpath.$param{'file'}.".json";
               if (-e $config_file) {
                    if (open(DAT,"<",$config_file)) {
                         flock(DAT, LOCK_EX);
                         $raw_data = <DAT>;
                         close(DAT);
                    } else {
                         close(DAT);
                         exit;
                    }
               }
               my $config_old = JSON::XS->new->decode($raw_data) if (defined $raw_data);
               my $config_new = JSON::XS->new->decode($param{'value'});
               if (-e $status_file) {
                    if (open(DAT,"<",$status_file)) {
                         flock(DAT, LOCK_EX);
                         $status_data = <DAT>;
                         close(DAT);
                    } else {
                         close(DAT);
                         exit;
                    }
               }
               my $status = JSON::XS->new->decode($status_data) if ($status_data);
               my $status_data = ({});
               foreach my $cf (@{$config_new->{'config'}}) {
                    if ($status && scalar(keys %$status)) {
                          $status_data->{$cf->{'id'}} = $status->{$cf->{'id'}} if (defined $status->{$cf->{'id'}});
                    }
                    delete $cf->{'status'};
                    delete $cf->{'status_msg'};
                    delete $cf->{'wtime'};
                    delete $cf->{'ltime'};
#                    $cf->{'ip'} = ($cf->{'iptype'} eq "auto" ? '' : $status_data->{$cf->{'id'}}->{'ip'});
                    $cf->{'ip'} = '' if ($cf->{'iptype'} eq "auto");
                    if ($cf->{'password'} ne "" && $cf->{'password'} ne "0:1:2:3:4:5:6:7") {
                         $cf->{'password'} = DDNSUpdater::CryptDecrypt->encrypt_data($cf->{'password'},'','');
                    } else {
                         @grep_ret = ();
                         @grep_ret = grep {$_->{'id'} eq $cf->{'id'}} @{$config_old->{'config'}};
                         next if (scalar(@grep_ret) == 0);
                         $cf->{'password'} = $grep_ret[0]{'password'};
                    }
               }
               $status_new = JSON::XS->new->encode($status_data) if ($status_data);
               $param{'value'} = JSON::XS->new->encode($config_new);
#               $param{'value'} =~ s/"id":"([0-9]+)"/"id":$1/g;
          } elsif ($param{'file'} eq "provider") {
               my $provider_new = JSON::XS->new->decode($param{'value'});
               foreach my $pv (@{$provider_new->{'provider'}}) {
                    delete $pv->{'display'};
                    delete $pv->{'wizard'};
               }
               $param{'value'} = JSON::XS->new->encode($provider_new);
          } elsif ($param{'file'} eq "protocols") {
               my $protocols_new = JSON::XS->new->decode($param{'value'});
               foreach my $pc (@{$protocols_new->{'protocols'}}) {
                    delete $pc->{'wizard'};
               }
               $param{'value'} = JSON::XS->new->encode($protocols_new);
          } elsif ($param{'file'} eq "options") {
               my $options = JSON::XS->new->decode($param{'value'});
               if (!$options->{'options'}[0]->{'debug_enabled'} && -s $debug_file != 0) {
                    unlink ($debug_file);
               }
          }
          if (open(DATA,">",$configpath.$param{'file'}.".json")) {
               flock(DATA, LOCK_EX);
               print DATA $param{'value'};
               close (DATA);
               if ($param{'file'} eq "config" && $status_new) {
                    if (open(STATUS,">",$status_file)) {
                         flock(STATUS, LOCK_EX);
                         print STATUS $status_new;
                         close (STATUS);
                         print '{"success":true}';
                    } else {
                         close (STATUS);
                         print '{"success":false}';
                    }
               } else {
                    print '{"success":true}';
               }
          } else {
               close (DATA);
               print '{"success":false}';
          }
          if ($param{'restart'} eq "true") {
               my $pid = get_pid($pidfile);
               exec("/bin/kill -1 $pid") if ($pid);
          }
     }
}
exit;
