#!/usr/bin/perl
#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       detectmodule.cgi                                     #
#  Description: a perl script to test protocol return codes          #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2015 by QTip                                         #
#  License:     GNU GPLv3 (see LICENSE)                              #
#  ----------------------------------------------------------------  #
#  Version:     0.4 - 14/10/2015                                     #
#  for more information check the changelog                          #
#********************************************************************#

$user = `/usr/syno/synoman/webman/modules/authenticate.cgi`;
$user =~ s/^\s+|\s+$//g;
exit if ($user eq '');

use strict;
use warnings;

use FindBin qw($Bin);
use File::Spec;
use lib ("$Bin/../../lib");

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use Data::Dumper;

## initialize vars #################
my $program = "ddud_md";
my $version = "0.4";
my $build_date = "20151014";
my @cpath = File::Spec->splitdir($Bin);
my $packagename = "$cpath[3]";
my $packagehome = "/var/packages/$packagename";
my $configpath = "$packagehome/etc";
my $modulpath = "DDNSUpdater::Module";
my $dsmbuild = `/bin/get_key_value /etc.defaults/VERSION buildnumber`;
chomp($dsmbuild);
our $useragent = $program.'/'.$version.'/'.$build_date;

$ENV{'PLIENT_HANDLER_PREFERENCE'} = "http:curl;https:curl";
$ENV{'PLIENT_CURL'} = ($dsmbuild >= 4458 ? "/usr/bin/curl" : "/usr/syno/bin/curl");
binmode(STDOUT,":encoding(UTF-8)") if ($ENV{'DDU_DEBUG'});

use DDNSUpdater::Functions;
use DDNSUpdater::Tools qw($ips $EOL $useragent);
use DDNSUpdater::Logger;
use DDNSUpdater::CryptDecrypt;

my (@parts,$part,$buffer,$value,$name,%param,$raw_data);

###################################

print "Content-type: application/json; charset=UTF-8\n\n";

if ($ENV{'REQUEST_METHOD'} eq "POST") {
     read(STDIN,$buffer,$ENV{"CONTENT_LENGTH"});
     @parts = split(/\&/,$buffer);
     foreach $part (@parts) {
          ($name,$value) = split(/=/,$part);
          $value =~ tr/+/ /;
          $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
          # prevent include of server side includes
          $value =~ s/<!--(.|\n)*-->//g;
          $param{$name} = $value;
     }
     if ($param{'action'} eq "apply") {
          my ($duoptions,$duchecksites,$duresponses,%rgroup);
          my ($updatevars,$updateurl,$provider,@protocol,$ipsys,$pdebug,$pinfo);

          $duoptions = DDNSUpdater::Functions->read_config($configpath."/options.json");
          $duchecksites = DDNSUpdater::Functions->read_config($configpath."/sites.json");
          $duresponses = DDNSUpdater::Functions->read_config($configpath."/responses.json");
          $ENV{'DDU_DEBUG_LOG'} = (exists $duoptions->{'options'}[0]{'debug_enabled'} && $duoptions->{'options'}[0]{'debug_enabled'} eq eTRUE) ? 1 : 0;

          my $ipdetect = $duoptions->{'options'}[0]{'ipdetect'};
          my @ipsystem = ($duoptions->{'options'}[0]{'ipsystem'} eq "ipv46") ? ('ipv4','ipv6') : $duoptions->{'options'}[0]{'ipsystem'};

          # build response group hash
          foreach my $rg (@{$duresponses->{'responses'}}) {
               push(@{$rgroup{$rg->{'responsegroup'}}},$rg->{'responsename'});
          }
#          DDNSUpdater::Functions->print_environment() if ($ENV{'DDU_DEBUG'});
          print $EOL."DEBUG: ===============================".$EOL if ($ENV{'DDU_DEBUG'});
          print "DEBUG: ***Debug start +++module detect+++ ***".$EOL.$EOL if ($ENV{'DDU_DEBUG'});
          debug("===============================") if ($ENV{'DDU_DEBUG_LOG'});
          debug("***Debug start module detect ***") if ($ENV{'DDU_DEBUG_LOG'});
          DDNSUpdater::Tools->get_ip($ipdetect,\@ipsystem,\@{$duchecksites->{'sites'}},$dsmbuild);

          my $scale_units = {'h' => 3600, 'm' => 60, 's' => 1};
          my $ducfg = ();
          my @gh = ();
          $updatevars = ();
          $updatevars->{'script'} = '';
          $updatevars->{'auth'} = {};
          $provider = ();
          @protocol = ();
          $updateurl = '';

          $ducfg = JSON::XS->new->decode($param{'config'});
          $provider = JSON::XS->new->decode($param{'provider'});
          @protocol = @{JSON::XS->new->decode($param{'protocol'})};
          $protocol[0]->{'module'} = $param{'module'};
          push(@gh,{'id' => 0, 'host' => $ducfg->{'host'}});
#print Dumper(\@gh);
#print Dumper($ducfg);
#print Dumper(@provider);
#print Dumper(@protocol);
#exit;
          $ipsys = ($ducfg->{'ipv6'}) ? 'ipv6' : 'ipv4';

          if ((defined $ducfg->{'ip'} && ($ducfg->{'ip'} eq "" || $ducfg->{'ip'} eq "0")) || !defined $ducfg->{'ip'}) {
               $ducfg->{'ip'} = ($ducfg->{'ipv6'} ? $ips->{'ipv6'} : $ips->{'ipv4'});
          }

          $pdebug = "host: ".$ducfg->{'host'}
                .", ip-system: ".$ipsys
                .", ip-set method: ".$ducfg->{'iptype'}
                .", update: ".($ducfg->{'iptype'} eq "auto"?$ips->{$ipsys}:$ducfg->{'ip'});
          print "DEBUG: --------------".$EOL if ($ENV{'DDU_DEBUG'});
          print "DEBUG: computed vars:".$EOL if ($ENV{'DDU_DEBUG'});
          print "DEBUG: ".$pdebug.$EOL if ($ENV{'DDU_DEBUG'});
          debug("--------------") if ($ENV{'DDU_DEBUG_LOG'});
          debug("computed vars:") if ($ENV{'DDU_DEBUG_LOG'});
          debug($pdebug) if ($ENV{'DDU_DEBUG_LOG'});

          DDNSUpdater::Tools->build_updatevars($ducfg,$updatevars,$provider,\@protocol);
#print Dumper($updatevars);

          debug("exec update call:") if ($ENV{'DDU_DEBUG_LOG'});
          print "DEBUG: exec update call:".$EOL if ($ENV{'DDU_DEBUG'});
          my ($error,$errortext,%result) = ('','','');
          my $ret = DDNSUpdater::Functions->require_class_by_string($modulpath.'::'.$protocol[0]->{'module'});
          if ($ret) {
               print "DEBUG: Unknown module ".$protocol[0]->{'module'}.", using next!".$EOL if ($ENV{'DDU_DEBUG'});
               debug("[CRITICAL] Unknown module '".$protocol[0]->{'module'}."', using next!") if ($ENV{'DDU_DEBUG_LOG'});
               exit;
          }
          ($error,$errortext,%result) = ($modulpath.'::'.$protocol[0]->{'module'})->update_protocol($updatevars,\@gh);
          print "DEBUG: ".$error." - ".$errortext.$EOL if ($ENV{'DDU_DEBUG'});
          print "DEBUG: ".Dumper(\%result).$EOL if ($ENV{'DDU_DEBUG'});
          print "DEBUG: --------------".$EOL if ($ENV{'DDU_DEBUG'});
          debug($error." - ".$errortext) if ($ENV{'DDU_DEBUG_LOG'});
          debug(\%result) if ($ENV{'DDU_DEBUG_LOG'});
          debug("--------------") if ($ENV{'DDU_DEBUG_LOG'});

          my @codefound_ok = ();
          my @codefound_nochg = ();
          my @codefound_tosoon = ();
          my $response = "nok";
          if ($error == 0) {
               @codefound_ok = grep {$result{'0'}{'code'} =~ /\Q$_\E/} @{$rgroup{'OK'}};
               @codefound_nochg = grep {$result{'0'}{'code'} =~ /\Q$_\E/} @{$rgroup{'NOCHG'}};
               @codefound_tosoon = grep {$result{'0'}{'code'} =~ /\Q$_\E/} @{$rgroup{'TOSOON'}};
               if (scalar @codefound_ok || scalar @codefound_nochg || scalar @codefound_tosoon) {
                    $response = "ok";
                    $pdebug = "status found: ".$result{'0'}{'code'}.", ip set: ".$ducfg->{'ip'};
               } else {
                    $pdebug = "status found: ".$result{'0'}{'code'};
               }
          } else {
               $pdebug = "error: ".$error.", errortext: ".$errortext.", '".$result{'0'}{'code'}."'";
          }
          print "DEBUG: ".$pdebug.$EOL if ($ENV{'DDU_DEBUG'});
          print "DEBUG: --------------".$EOL if ($ENV{'DDU_DEBUG'});
          debug($pdebug) if ($ENV{'DDU_DEBUG_LOG'});
          debug("--------------") if ($ENV{'DDU_DEBUG_LOG'});
          print '{"success":true,"error":"'.$error.'","errortext":"'.$errortext.'","code":"'.$result{'0'}{'code'}.'","result":"'.$response.'"}';
     }
     print $EOL."DEBUG: ***Debug end +++module detect+++ ***".$EOL if ($ENV{'DDU_DEBUG'});
     print "DEBUG: ===============================".$EOL if ($ENV{'DDU_DEBUG'});
     debug("***Debug end +++module detect+++ ***") if ($ENV{'DDU_DEBUG_LOG'});
     debug("===============================") if ($ENV{'DDU_DEBUG_LOG'});
}
exit;
#          sleep(3);
#          if ($param{'module'} eq "default") {
#               print '{"success":true,"result":"ok"}';
#          } else {
#               print '{"success":true,"result":"nok"}';
#          }
