#!/usr/bin/perl
#*******************************************************************#
#  DDNS Updater - 3rdparty DDNS host updater                        #
#  Modul:       export.cgi                                          #
#  Description: a perl script to export and download a file         #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2014-2016 by QTip                                   #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.51 - 17/01/2016                                   #
#  for more information check the changelog                         #
#*******************************************************************#

$user = `/usr/syno/synoman/webman/modules/authenticate.cgi`;
$user =~ s/^\s+|\s+$//g;
exit if ($user eq '');

use strict;
use warnings;

use FindBin qw($Bin);
use File::Spec;
use lib ("$Bin/../../lib");

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use POSIX qw(strftime);
use JSON::XS;
use DDNSUpdater::Logger;
#use Data::Dumper;

## initialize vars #################
my @path = File::Spec->splitdir($Bin);
my $exportpath = "/var/packages/$path[3]/target/tmp";
my $debuglogfile = "/var/packages/$path[3]/target/log/ddud_debug.log";
my $configpath = "/var/packages/$path[3]/etc";
my $compress_cmd = "/bin/tar czf ";
unless (-d $exportpath) {mkdir($exportpath,0777) || die "Could not create export dir ($!)\n\n";}

my (@filebuffer,$result,$download_file,$dl_file,$packagedata);

if ($ENV{'REQUEST_METHOD'} eq "POST") {
     # clear tmp dir
     my $cgi = CGI->new();
     my %param = $cgi->Vars();
     if ($param{'action'} eq "export" && $param{'type'} ne "") {
          print $cgi->header(-type => 'text/html');
          my $now = strftime "%Y%m%d%H%M", localtime;
          if ($param{'type'} eq "parts") {
               $dl_file = "ddnsupdater2_backup_".$now.".tgz";
               $download_file = $exportpath."/".$dl_file;
               $packagedata = JSON::XS->new->decode($param{'value'});
               my $list = join(".json ",@{$packagedata}).".json";
               $result = `cd $configpath && $compress_cmd $download_file $list >&1`; # --exclude=status.*
               if ($? == 0) {
                    print '{"success":true,"file":"'.$dl_file.'"}';
                    info("File '".$dl_file."' (".join(', ',@{$packagedata}).") successfully exported!");
               } else {
                    print '{"success":false,"msg":"StatusExportFailedText"}';
                    error("There was an error during export of the file '$dl_file'!");
               }
          } elsif ($param{'type'} eq "package") {
               $dl_file = "ddnsupdater2_".$param{'type'}."_".$now.".tgz";
               $download_file = $exportpath."/".$dl_file;
               $param{'value'} =~ tr/+/ /;
               $param{'value'} =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
               # prevent include of server side includes
               $param{'value'} =~ s/<!--(.|\n)*-->//g;
               $packagedata = JSON::XS->new->decode($param{'value'});
#print Dumper($packagedata);
               my $error = 0;
               my $compress_files = '';
               for my $root (keys %{$packagedata}) {
#print Dumper($packagedata->{$file});
                    next unless (exists $packagedata->{$root});
                    $compress_files .= $root.".json ";
                    my $contents = JSON::XS->new->encode($packagedata->{$root});
                    if (open(DATA,">",$exportpath."/".$root.".json")) {
                         print DATA '{"'.$root.'":'.$contents.'}';
                         close (DATA);
                    } else {
                         $error = 1;
                    }
               }
               if ($error) {
                    print '{"success":false,"msg":"StatusExportFailedText"}';
                    error("There was an error during export of the file '$dl_file'!");
               } else {
                    $result = `cd $exportpath && $compress_cmd $download_file $compress_files >&1`;
                    if ($? == 0) {
                         print '{"success":true,"file":"'.$dl_file.'"}';
                         info("File '$dl_file' successfully exported!");
                    } else {
                         print '{"success":false,"msg":"StatusExportFailedText"}';
                         error("There was an error during export of the file '$dl_file'!");
                    }
               }

          } else {
               $dl_file = "ddnsupdater2_".$param{'type'}."_".$now.".json";
               $download_file = $exportpath."/".$dl_file;
               $param{'value'} =~ tr/+/ /;
               $param{'value'} =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
               # prevent include of server side includes
               $param{'value'} =~ s/<!--(.|\n)*-->//g;
               if (open(DATA,">",$download_file)) {
                    print DATA $param{'value'};
                    close (DATA);
                    print '{"success":true,"file":"'.$dl_file.'"}';
                    info("File '$dl_file' successfully exported!");
               } else {
                    print '{"success":false,"msg":"StatusExportFailedText"}';
                    error("There was an error during export of the file '$dl_file'!");
               }
          }
     }
} else {
     my $cgi = CGI->new();
     my %param = $cgi->Vars();
     if ($param{'action'} eq "download" && $param{'mode'} ne "" && $param{'file'} ne "") {
          $download_file = ($param{'mode'} eq "export") ? $exportpath."/".$param{'file'} : $debuglogfile;
          exit unless (-e $download_file);
          my $filesize = -s $download_file;
          open(DLFILE,"<",$download_file) || Error('open', 'file');
          @filebuffer = <DLFILE>;
          close (DLFILE) || Error ('close', 'file');
          print "Cache-Control: maxage=120\n";
          print "Expires: 0";
          print "Pragma: public\n";
          print "Content-type:application/force-download\n";
          print "Content-Transfer-Encoding: Binary\n";
          print "Content-Type:application/octet-stream\n";
          print "Content-Disposition: attachment; filename=\"$param{'file'}\"\n";
          print "Content-Length: $filesize\n\n";
          print @filebuffer;
     }
}
exit;
