#!/usr/bin/perl
#*******************************************************************#
#  DDNS Updater - 3rdparty DDNS host updater                        #
#  Modul:       import.cgi                                          #
#  Description: a perl script to import a file                      #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2014-2015 by QTip                                   #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.4 - 08/04/2015                                    #
#  for more information check the changelog                         #
#*******************************************************************#

$user = `/usr/syno/synoman/webman/modules/authenticate.cgi`;
$user =~ s/^\s+|\s+$//g;
exit if ($user eq '');

use strict;
use warnings;

use FindBin qw($Bin);
use File::Spec;
use lib ("$Bin/../../lib");

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use Fcntl qw(:flock);
use File::Basename;
use DDNSUpdater::Logger;
use DDNSUpdater::Import;
use DDNSUpdater::Functions;
#use Data::Dumper;

sub upload_file{
     my ($filename,$filehandle,$path) = @_;
     my $target = $path."/".$filename;
     binmode $filehandle;
     open(TARGET,">",$target) or die $!;
     binmode TARGET;
     my $buffer;
     while(read $filehandle,$buffer,1024){
          print TARGET $buffer;
     }
     close TARGET;
     return ($? > 255) ? ($? % 256) : $?;
}
sub test_archive {
     my $file = shift;

     `tar -tzf "$file" 2> /dev/null`;
     return $? > 0 ? 0 : 1;
}

sub extract_archive {
     my $file = shift;
     my $dest = shift;

     unless (-d "$dest") {mkdir("$dest",0755) || die "Could not create archive dir ($!)\n\n";}
     chdir($dest);
     `tar -xzf "$file" 2> /dev/null`;
     return $? > 0 ? 0 : 1;
}

my @path = File::Spec->splitdir($Bin);
my $importpath = "/var/packages/$path[3]/target/tmp";
my $extractpath = "/var/packages/$path[3]/target/tmp/archive";
my $configpath = "/var/packages/$path[3]/etc";
my $extact_cmd = "/bin/tar czf ";
my $safe_filename_characters = "a-zA-Z0-9_.-";
unless (-d $importpath) {mkdir($importpath,0777) || die "Could not create import dir ($!)\n\n";}
if (-d $importpath) {unlink glob($importpath."/*");};
if (-d $extractpath) {unlink glob($extractpath."/*");};

my $dsmmajor = `/bin/get_key_value /etc.defaults/VERSION majorversion`;
chomp($dsmmajor);

my $max_filesize = ($dsmmajor >= 5) ? `/bin/get_key_value /etc/php/php.ini upload_max_filesize` : `/bin/get_key_value /usr/syno/etc/php.ini upload_max_filesize`;
chomp($max_filesize);
$max_filesize = 1024 * 1024 * substr($max_filesize,0,-1);
$CGI::POST_MAX = $max_filesize;

my ($filehandle,$filename,$filesize,$ret,$upload_file,$retjson,$retold,$clean,$upgrade,$sum);
my $archive = 0;
my $result = {'result' => []};

if ($ENV{'REQUEST_METHOD'} eq "POST") {
     my $cgi = CGI->new();
     my %param = $cgi->Vars();
     if ($param{'action'} eq "restore" && $param{'duRestoreConfig'} ne "") {
          print $cgi->header(-type => 'text/html');
          $filehandle = $cgi->upload('duRestoreConfig');
          $clean = (defined $param{'duRestoreClean'} && $param{'duRestoreClean'} == 1)?1:0;
          $upgrade = (defined $param{'duRestoreUpgrade'} && $param{'duRestoreUpgrade'} == 1)?1:0;
#          $upgrade = 1;
          $filename = $param{'duRestoreConfig'};
          my ($fname,$fpath,$fextension) = fileparse($filename,'..*');
          $filename = $fname.$fextension;
          $filename =~ tr/ /_/;
          $filename =~ s/[^$safe_filename_characters]//g;
          $filesize = -s $filehandle;
          if ($max_filesize == 0 || ($max_filesize > 0 && $filesize > $max_filesize)) {
               print '{"success":false, "msg":"StatusImportSizeText", "size":'.($max_filesize/(1024*1024)).'}';
               error("File '$filename' exceeds the max. size of ".($max_filesize/(1024*1024))." MB");
               exit;
          }
          if ($filename =~ /^([$safe_filename_characters]+)$/) {
               $filename = $1;
          } else {
               print '{"success":false, "msg":"StatusImportFnameInvalidCharsText"}';
               error("Filename of '$filename' contains invalid characters!");
               exit;
          }
          print $cgi->start_html();
          $ret = upload_file($filename,$filehandle,$importpath);
          print $cgi->end_html();
          if ($ret == 0) {
               if (test_archive($importpath."/".$filename)) {
                    # tgz archive file
                    $ret = extract_archive($importpath."/".$filename,$extractpath);
                    if ($ret) {
                         $importpath = $extractpath;
                    } else {
                         print '{"success":false, "msg":"StatusImportArchiveErrorText"}';
                         error("The archive '$filename' is broken!");
                         exit;
                    }
               }
               my @imp = glob($importpath."/*");
               my $list;
#print Dumper(\@imp);
               foreach my $file (@imp) {
                    if (-f $file) {
#print $file;
                         $sum = ();
                         $retjson = 0;
                         $retold = 0;
                         ($retjson,$sum) = DDNSUpdater::Import->import($configpath,$file,$clean,$upgrade);
                         ($retold,$sum) = DDNSUpdater::Import->convert($configpath,$file,$clean) if ($retjson == 1);
                         if ($retjson == 1 && $retold) {
                              $sum->{'msg'} = "StatusImportNoDDU12Text";
                              $sum->{'type'} = 0;
                         }
                         $list .= $sum->{'file'}.', ' if ($retjson == 0);
                         push(@{$result->{'result'}},\%$sum);
                    }
               }
               $list = substr($list,0,-2);
               my $missing = DDNSUpdater::Functions->check_data_consistency($configpath,'');
               my ($prots,$provs);
#print Dumper($missing);
               $result->{'warnings'} = $missing->{'warnings'};
               if (scalar(@{$missing->{'protocols'}})) {
                    $prots = join(", ",@{$missing->{'protocols'}});
                    warning("Missing protocols on import: ".$prots);
               } elsif (scalar(@{$missing->{'provider'}})) {
                    $provs = join(", ",@{$missing->{'provider'}});
                    warning("Missing provider on import: ".$provs);
               }
               unlink $configpath."/status.json" if ($clean && -e $configpath."/status.json");
               $result->{'success'} = do {bless \(my $dummy = 1), "JSON::XS::Boolean"};
#print Dumper(\%$sum);
               my $json = JSON::XS->new->encode($result);
               print $json;
               info("File '".$filename."' (".$list.") successfully imported!");
          } else {
               print '{"success":false, "msg":"StatusImportFailedText"}';
               error("There was an error during upload of the file '$filename'!");
          }
     }
}
exit;
