#!/usr/bin/perl
#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       updater.cgi                                          #
#  Description: a perl script to update data from online repository  #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2014-2016 by QTip                                    #
#  License:     GNU GPLv3 (see LICENSE)                              #
#  ----------------------------------------------------------------  #
#  Version:     0.52 - 29/05/2016                                    #
#  for more information check the changelog                          #
#********************************************************************#

$user = `/usr/syno/synoman/webman/modules/authenticate.cgi`;
$user =~ s/^\s+|\s+$//g;
exit if ($user eq '');

use strict;
use warnings;

use FindBin qw($Bin);
use lib ("$Bin/../../lib");

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use JSON::XS;
#use Encode qw(decode_utf8);
use Fcntl qw(:flock);
use Data::Dumper;

## initialize vars #################
my $program = "dduu";
my $version = "0.51";
my $build_date = "20160116";
my @path = File::Spec->splitdir($Bin);
my $packagehome = "/var/packages/$path[3]";
my $packagetarget = "$packagehome/target";
my $configpath = "$packagehome/etc";
my $pidfile = "/var/run/ddud.pid";
my $updateurl = 'https://www.cphub.net/ddu/ddu_updater.php';
my @checkdirs = ('target/lib/DDNSUpdater/Module','etc'); # relative to $packagetarget
my @updateroots = ('modules','provider','protocols','responses','sites');
our $useragent = $program.'/'.$version.'/'.$build_date;
my $dsmbuild = `/bin/get_key_value /etc.defaults/VERSION buildnumber`;
chomp($dsmbuild);

if ($dsmbuild >= 4458) {
     require Digest::SHA;
     Digest::SHA->import(qw{sha256_hex});
}

my (@parts,$part,$buffer,$value,$name,%param,@grep_ret);
my ($error,$errortext);
my $locallist = ();
my $listdata = ();
my $updatedata = ();
my @difflist = ();
my ($localjson,$listjson,$updatejson,$diffjson,$moduledata);
my $result = {'result' => []};
my $auth = {'login' => 'dduuuser', 'password' => 'VYt2RKvN2wPd3RQPTRdtwBEtIRRVWZgBjSeOhUpn'};

$ENV{'PLIENT_HANDLER_PREFERENCE'} = "http:curl;https:curl";
$ENV{'PLIENT_CURL'} = ($dsmbuild >= 4458 ? "/usr/bin/curl" : "/usr/syno/bin/curl");
$ENV{'DDU_DEBUG'} ||= 0;
binmode(STDOUT,":encoding(UTF-8)") if ($ENV{'DDU_DEBUG'});

use DDNSUpdater::Logger;
use DDNSUpdater::Import;
use DDNSUpdater::Functions;
use DDNSUpdater::Tools qw($EOL $useragent);

# generate provider and protocol list
sub _generate_pp_list {
     my $path = shift;

     my $jsondata = DDNSUpdater::Functions->read_config($path);
     my ($root) = keys %{$jsondata};
     foreach my $js (@{$jsondata->{$root}}) {
          if ($root eq "provider" || $root eq "protocols") {
               push (@{$locallist->{$root}},{'n' => $js->{'name'}, 'v' => $js->{'version'}, 'f' => $js->{'ftype'}});
          } elsif ($root eq "responses") {
               push (@{$locallist->{$root}},{'n' => $js->{'responsename'}, 'v' => 1, 'f' => $js->{'ftype'}});
          } elsif ($root eq "sites") {
               push (@{$locallist->{$root}},{'n' => $js->{'url'}, 'v' => 1, 'f' => $js->{'ftype'}});
          }
     }
}
sub _read_module {
     my $path = shift;

     my $ret = `/bin/grep "Version:" $path | /usr/bin/cut -d " " -f8,10`;
     my ($v,$l) = split(" ",$ret);
     return $v;
}
sub _get_filelist {
     my $path = shift;

     my $root;
     opendir(my $handle, $packagehome."/".$path) or die $!;
     while(my $e = readdir($handle)) {
          next if ($e =~ /^(\.\.?|\.svn|tmp|bak|db|log|.*\.conf|\@eaDir|testing|extjs|config.json|groups.json|options.json|modules.json|status.json)$/);
#          if (-d $packagehome."/".$path."/".$e) {
               # scan subdir
#               _get_filelist($path."/".$e);
          if ($e =~ /.*\.json$/) {
               _generate_pp_list($packagehome."/".$path."/".$e);
          } else {
               $root = ($e =~ /.*\.pm$/ ? 'modules' : substr($e,0,-5));
#               push (@{$locallist->{$root}},{'n' => $e, 'v' => _sha256hex_file($packagetarget."/".$path."/".$e), 'f' => 'buildin'});
               push (@{$locallist->{$root}},{'n' => $e, 'v' => _read_module($packagehome."/".$path."/".$e), 'f' => 'buildin'});
          }
     }
     closedir($handle);
}
sub _generate_local_list {
     foreach my $ud (@checkdirs) {
          _get_filelist($ud);
     }
}
sub _compare_version_list {
     my $root = shift;

     foreach my $ul (@{$listdata->{$root}}) {
          @grep_ret = ();
          @grep_ret = grep {$_->{'n'} eq $ul->{'n'}} @{$locallist->{$root}};
          if (scalar(@grep_ret) == 0) {
               push (@difflist,{'type' => $root, 'name' => $ul->{'n'}, 'version' => $ul->{'v'}, 'module' => $ul->{'m'}, 'status' => 'DUNewText'});
          } else {
               @grep_ret = ();
               if ($ul->{'n'} =~ /.*\.pm$/) {
                    @grep_ret = grep {$_->{'n'} eq $ul->{'n'} && $_->{'v'} < $ul->{'v'}} @{$locallist->{$root}};
               } else {
                    @grep_ret = grep {$_->{'n'} eq $ul->{'n'} && $_->{'v'} < $ul->{'v'}} @{$locallist->{$root}};
               }
               if (scalar(@grep_ret) && ($grep_ret[0]->{'f'} eq "" || $grep_ret[0]->{'f'} eq "addon" || $grep_ret[0]->{'f'} eq "buildin")) {
                    push (@difflist,{'type' => $root, 'name' => $ul->{'n'}, 'version' => $ul->{'v'}, 'module' => $ul->{'m'}, 'status' => 'DUUpdateText'});
               } else {
                    next;
               }
          }
     }
}
sub _sha256_hex {
     my $value = shift;

     if ($dsmbuild >= 4458) {
          return sha256_hex($value);
     } else {
          my $sha256 = `/bin/echo -n "$value" | /usr/syno/bin/openssl dgst -sha256 2> /dev/null | /usr/bin/cut -d ' ' -f2`;
          chomp $sha256;
          return $sha256;
     }
}
###################################

print "Content-type: application/json; charset=UTF-8\n\n";

if ($ENV{'REQUEST_METHOD'} eq "POST") {
     read(STDIN,$buffer,$ENV{"CONTENT_LENGTH"});
     @parts = split(/\&/,$buffer);
     foreach $part (@parts) {
          ($name,$value) = split(/=/,$part);
          $value =~ tr/+/ /;
          $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
          # prevent include of server side includes
          $value =~ s/<!--(.|\n)*-->//g;
          $param{$name} = $value;
     }
     if ($param{'action'} eq "list" && $param{'type'} ne "") {
          ($error,$errortext,$listjson) = DDNSUpdater::Tools->api_request($updateurl,0,1,$auth,'POST',{'action'=>'list','secret'=>$param{'type'}},'diskstation');
          if ($error == 0 || $errortext eq "http:404") {
#print $listjson;
#exit;
               if ($listjson =~ m/^({.*?})$/) {
                    $listdata = JSON::XS->new->decode($listjson);
#print Dumper($listdata);
#exit;

                    _generate_local_list();
#print Dumper($locallist);
#exit;
                    foreach my $rl (sort keys %{$listdata}) {
                         _compare_version_list($rl);
                    }
                    #print Dumper(\@difflist) if (scalar(@difflist));
                    $diffjson = JSON::XS->new->encode(\@difflist);
#                    print $diffjson;
                    print '{"success":true,"msg":"'.$errortext.'","data":'.$diffjson.'}';
               } else {
                    print '{"success":true,"msg":"'.$errortext.'","data":[]}';
               }
          } else {
               print '{"success":false,"msg":"'.$errortext.'","data":[]}';
          }
     } elsif ($param{'action'} eq "update" && $param{'type'} ne "" && $param{'json'} ne "") {
          # here api call...
          my ($sum,$retjson,$uddata,$udjson);
          my $dumodules = ();
          ($error,$errortext,$updatejson) = DDNSUpdater::Tools->api_request($updateurl,0,1,$auth,'POST',{'action'=>'update','secret'=>$param{'type'},'json'=>$param{'json'}},'diskstation');
#print $updatejson;
#exit;
          if ($error == 0) {
               if ($updatejson =~ m/^({.*?})$/) {
#                    print $updatejson;
                    $updatedata = JSON::XS->new->decode($updatejson);
#print Dumper($updatedata);
                    foreach my $uj (keys %{$updatedata}) {
                         $sum = ();
                         if ($uj eq "modules") {
                              $sum->{'file'} = 'modules';
                              $sum->{'count'} = scalar(@{$updatedata->{$uj}});
                              $sum->{'imported'} = 0;
                              foreach my $m (@{$updatedata->{$uj}}) {
#print "$m->{'file'} - $m->{'hash'}\n";
                                   if ($m->{'hash'} ne "0") {
                                        ($error,$errortext,$moduledata) = DDNSUpdater::Tools->api_request($updateurl,0,1,$auth,'POST',{'action'=>'update','secret'=>$param{'type'},'module'=>$m->{'file'}},'diskstation');
#print "$moduledata\n";
#exit;
                                        if ($error == 0) {
                                             chomp ($moduledata);
                                             if ($m->{'hash'} eq _sha256_hex($moduledata)) {
                                                  if (open SMODULE,">", $packagetarget."/lib/DDNSUpdater/Module/".$m->{'file'}) { # <<<< fr Release ndern
                                                       print SMODULE $moduledata;
                                                       close SMODULE;
                                                       $sum->{'imported'}++;
                                                  } else {
                                                       $sum->{'msg'} = "StatusSaveModuleFailedText";
                                                  }
                                             } else {
                                                  $sum->{'msg'} = "StatusHashNotEqualText";
                                             }
                                        } else {
                                             $sum->{'msg'} = "StatusModuleNotFoundText";
                                        }
                                   } else {
                                        $sum->{'msg'} = "StatusHashEmptyText";
                                   }
                              }
                         } else {
                              $udjson = ();
                              $retjson = 0;
                              $uddata = {$uj => $updatedata->{$uj}};
                              $udjson = JSON::XS->new->encode($uddata);
                              ($retjson,$sum) = DDNSUpdater::Import->import($configpath,'',0,1,$udjson);
                         }
                         push(@{$result->{'result'}},\%$sum);
#print "$udjson\n";
                    }
                    # generate new modules list
                    _get_filelist($checkdirs[0]);
                    foreach my $lm (@{$locallist->{'modules'}}) {
                         push (@{$dumodules->{'modules'}},substr($lm->{'n'},0,-3)) if (substr($lm->{'n'},0,1) ne "#");
                    }
#print Dumper($dumodules);
                    my $json_modules = JSON::XS->new->encode($dumodules);
#print "$json_modules\n";
#exit;
                    if (open(MODULES,">","/var/packages/ddnsupdater2/etc/modules.json")) {
                         flock(MODULES, LOCK_EX);
                         print MODULES $json_modules;
                         close (MODULES);
                    } else {
                         close (MODULES);
                    }
                    my $missing = DDNSUpdater::Functions->check_data_consistency($configpath,'');
                    my ($prots,$provs);
#print Dumper($missing);
                    $result->{'warnings'} = $missing->{'warnings'};
                    if (scalar(@{$missing->{'protocols'}})) {
                         $prots = join(", ",@{$missing->{'protocols'}});
                         warning("Missing protocols on import: ".$prots);
                    } elsif (scalar(@{$missing->{'provider'}})) {
                         $provs = join(", ",@{$missing->{'provider'}});
                         warning("Missing provider on import: ".$provs);
                    }
                    $result->{'success'} = do {bless \(my $dummy = 1), "JSON::XS::Boolean"};
#print Dumper(\%$sum);
                    my $json = JSON::XS->new->encode($result);
                    print $json;
                    info("Online update performed successfully!");
                    my $pid = DDNSUpdater::Functions->read_pid($pidfile);
                    exec("/bin/kill -1 $pid") if ($pid);
                    exit;
               } else {
                    print '{"success":false, "msg":"StatusUpdateFailedText"}';
                    error("There was an error during online update!");
                    exit;
               }
          } else {
               print '{"success":false,"msg":"'.$errortext.'"}';
               exit;
          }
     }
}
#exit;
