#!/usr/bin/perl
#*******************************************************************#
#  DDNS Updater - 3rdparty DDNS host updater                        #
#  Modul:       upgrade.cgi                                         #
#  Description: a perl script to upgrade from DDNS Updater 1.xx     #
#  Author:      QTip from the german Synology support forum         #
#  Copyright:   2015 by QTip                                        #
#  License:     GNU GPLv3 (see LICENSE)                             #
#  ---------------------------------------------------------------  #
#  Version:     0.2 - 11/04/2015                                    #
#  for more information check the changelog                         #
#*******************************************************************#

$user = `/usr/syno/synoman/webman/modules/authenticate.cgi`;
$user =~ s/^\s+|\s+$//g;
exit if ($user eq '');

use strict;
use warnings;

use FindBin qw($Bin);
use File::Spec;
use lib ("$Bin/../../lib");

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use Fcntl qw(:flock);
use DDNSUpdater::Logger;
use DDNSUpdater::Import;

my @path = File::Spec->splitdir($Bin);
my $configpath = "/var/packages/$path[3]/etc";
my $oldconfig = "/var/packages/ddnsupdater/target/ddclient.conf";

my (@parts,$part,$buffer,$value,$name,%param,$retold,$sum);
my $result = {'result' => []};

print "Content-type: application/json; charset=UTF-8\n\n";

if ($ENV{'REQUEST_METHOD'} eq "POST") {
     read(STDIN,$buffer,$ENV{"CONTENT_LENGTH"});
     @parts = split(/\&/,$buffer);
     foreach $part (@parts) {
          ($name,$value) = split(/=/,$part);
          $value =~ tr/+/ /;
          $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
          # prevent include of server side includes
          $value =~ s/<!--(.|\n)*-->//g;
          $param{$name} = $value;
     }
     if ($param{'action'} eq "upgrade") {
          if (-f $oldconfig) {
               $sum = ();
               $retold = 0;
               ($retold,$sum) = DDNSUpdater::Import->convert($configpath,$oldconfig,0);
               push(@{$result->{'result'}},\%$sum);
          }
          my $missing = DDNSUpdater::Functions->check_data_consistency($configpath,'');
          my ($prots,$provs);
#print Dumper($missing);
          $result->{'warnings'} = $missing->{'warnings'};
          if (scalar(@{$missing->{'protocols'}})) {
               $prots = join(", ",@{$missing->{'protocols'}});
               warning("Missing protocols on import: ".$prots);
          } elsif (scalar(@{$missing->{'provider'}})) {
               $provs = join(", ",@{$missing->{'provider'}});
               warning("Missing provider on import: ".$provs);
          }
          $result->{'success'} = do {bless \(my $dummy = 1), "JSON::XS::Boolean"};
          my $json = JSON::XS->new->encode($result);
          print $json;
          info("DDNS Updater 1.xx host configuration successfully migrated!");
     }
}
exit;
