#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       Functions.pm                                         #
#  Description: misc perl functions                                  #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2014-2016 by QTip                                    #
#  License:     BSD-style license (see LICENSE)                      #
#  ----------------------------------------------------------------  #
#  Version:     0.43 - 05/05/2016                                    #
#  for more information check the changelog                          #
#********************************************************************#

package DDNSUpdater::Functions;

use strict;
use warnings;

use base qw(Exporter);

our $VERSION     = '0.43';
our @EXPORT      = qw(eTRUE eFALSE);
our @EXPORT_OK   = qw(trim in_array is_array array_diff print_environment read_config md5hex_file convert_iso2syno convert_syno2iso get_language
                      read_pid write_pid unlink_pid require_class_by_string remove_HTML convert_ynof2bool sendnotify check_data_consistency);

use JSON::XS;
use Fcntl qw(:flock);
use HTML::TokeParser::Simple;
use Encode qw(encode_utf8);
#use Data::Dumper;

# real true | false, compatible with encode_json decode_json for later (we don't want field:false... will be field:0...)
# http://stackoverflow.com/questions/1036347/how-do-i-use-boolean-variables-in-perl
use constant {
     eTRUE  =>  bless( do{\(my $o = 1)}, 'JSON::XS::Boolean' ),
     eFALSE =>  bless( do{\(my $o = 0)}, 'JSON::XS::Boolean' )
};

sub trim {
     my $class = shift;

     @_ = $_ if not @_ and defined wantarray;
     @_ = @_ if defined wantarray;
     for (@_ ? @_ : $_) { s/^\s+//, s/\s+$// }
     return wantarray ? @_ : $_[0] if defined wantarray;
}
sub in_array {
     my $class = shift;
     my ($needle,$haystack) = @_;

     foreach my $value ($haystack) {
         return 1 if $value eq $needle;
     }
     return 0;
}
sub is_array {
     my $class = shift;
     my $ref = shift;

     return 0 unless ref $ref;
     if (ref($ref) eq "ARRAY") {
          return 1;
     } else {
          return 0;
     }
}
sub last_n_elements {
     my $class = shift;
     my ($n,$haystack) = @_;

     my @last_n = ($n >= @{$haystack}) ? @{$haystack} : @{$haystack}[-$n..-1];
     return \@last_n;
}
sub array_diff {
     my $class = shift;
     my @a = @{(shift)};
     my @b = @{(shift)};

     my %in_bl = map {$_ => 1} @b;
     my @rest = grep {not $in_bl{$_}} @a;
     return @rest;
}
sub print_environment {
     my $class = shift;

     print "\nEnvironment:\n";
     print "     $_ $ENV{$_}\n" for (keys %ENV);
     print "\n";
}
sub read_config {
     my $class = shift;
     my $configfile = shift;

     my $raw_data;
     if (-e $configfile) {
          if (open(DAT,"<",$configfile)) {
               flock(DAT, LOCK_EX);
               $raw_data = <DAT>;
               close(DAT);
          } else {
               close(DAT);
               error("Can not load $configfile!");
               return $raw_data;
          }
     } else {
          return $raw_data;
     }
     my $data = (JSON::XS->new->decode($raw_data)) if ($raw_data);
     return $data;
}
sub md5hex_file {
     my $class = shift;
     my $fname = shift;

     my $md5file = "";
     if (open FILE,"<",$fname) {
          my $md5file = Digest::MD5->new;
          $md5file->addfile (*FILE);
          my $hash = $md5file->hexdigest;
          close (FILE);
          return $hash;
     } else {
          return 0;
     }
}
sub convert_iso2syno {
     my $class = shift;
     my $lang = shift;

     my %iso2syno;
     $iso2syno{"de"} = "ger";  #german
     $iso2syno{"en"} = "enu";  #english US
     $iso2syno{"zh"} = "chs";  #chinese simplified
     $iso2syno{"cs"} = "csy";  #czech
     $iso2syno{"jp"} = "jpn";  #japan
     $iso2syno{"ko"} = "krn";  #korean
     $iso2syno{"da"} = "dan";  #danish
     $iso2syno{"fr"} = "fre";  #french
     $iso2syno{"it"} = "ita";  #italian
     $iso2syno{"nl"} = "nld";  #dutch
     $iso2syno{"no"} = "nor";  #norwegian
     $iso2syno{"pl"} = "plk";  #polish
     $iso2syno{"ru"} = "rus";  #russian
     $iso2syno{"sp"} = "spn";  #spanish
     $iso2syno{"sv"} = "sve";  #swedish
     $iso2syno{"hu"} = "hun";  #hungarian
     $iso2syno{"tr"} = "trk";  #turkish
     $iso2syno{"pt"} = "ptg";  #portuguese european
     return $iso2syno{$lang};
}
sub convert_syno2iso {
     my $class = shift;
     my $lang = shift;

     my %syno2iso;
     $syno2iso{"ger"} = "de";  #german
     $syno2iso{"enu"} = "en";  #english US
     $syno2iso{"chs"} = "zh";  #chinese simplified
     $syno2iso{"csy"} = "cs";  #czech
     $syno2iso{"jpn"} = "jp";  #japan
     $syno2iso{"krn"} = "ko";  #korean
     $syno2iso{"dan"} = "da";  #danish
     $syno2iso{"fre"} = "fr";  #french
     $syno2iso{"ita"} = "it";  #italian
     $syno2iso{"nld"} = "nl";  #dutch
     $syno2iso{"nor"} = "no";  #norwegian
     $syno2iso{"plk"} = "pl";  #polish
     $syno2iso{"rus"} = "ru";  #russian
     $syno2iso{"spn"} = "sp";  #spanish
     $syno2iso{"sve"} = "sv";  #swedish
     $syno2iso{"hun"} = "hu";  #hungarian
     $syno2iso{"trk"} = "tr";  #turkish
     $syno2iso{"ptg"} = "pt";  #portuguese european
     return $syno2iso{$lang};
}
sub get_language {
     my $class = shift;
     my $user = shift;

     my $dsmlanguage = `/bin/get_key_value /etc/synoinfo.conf language`;
     chomp($dsmlanguage);

     my @bl = ($ENV{HTTP_ACCEPT_LANGUAGE} ne "") ? split(',',$ENV{HTTP_ACCEPT_LANGUAGE}) : '';
     my $browserlanguage = (scalar(@bl)) ? DDNSUpdater::Functions->convert_iso2syno(substr($bl[0],0,2)) : $dsmlanguage;

     $dsmlanguage = ($dsmlanguage eq "def") ? $browserlanguage : $dsmlanguage;

     my ($raw_data,$json_data,$jdata);
     my $usersettings_file = "/usr/syno/etc/preference/".$user."/usersettings";
     if (-e $usersettings_file) {
          if (open(DAT,"<",$usersettings_file)) {
               $raw_data = <DAT>;
               close(DAT);
               $jdata = JSON::XS->new->utf8;
               $json_data = $jdata->decode($raw_data);
               if (${$json_data}{Personal}->{lang} && ${$json_data}{Personal}->{lang} ne 'def') {
                    $dsmlanguage = ${$json_data}{Personal}->{lang};
               }
          }
     }
     return $dsmlanguage;
}
sub read_pid {
     my $class = shift;
     my $pidfile = shift;

     if (-e $pidfile) {
          local *FD;
          if (!open(FD, "<",$pidfile)) {
               return 0;
        	} else {
               my $PID = <FD>;
               close(FD);
               return $PID;
          }
     }
     return 0;
}
sub write_pid {
     my $class = shift;
     my $pidfile = shift;

     if ($pidfile) {
          local *FD;
          if (!open(FD, ">", $pidfile)) {
               error("Cannot create file $pidfile");
          } else {
               printf FD "$$\n";
               close(FD);
          }
    }
}
sub unlink_pid {
     my $class = shift;
     my $pidfile = shift;

     unlink $pidfile if (-e $pidfile);
}
# http://stackoverflow.com/questions/1917261/how-can-i-dynamically-include-perl-modules-without-using-eval
sub require_class_by_string {
     my $class = shift;
     my $modul = shift;

     use FindBin qw($Bin);
     (my $file = $modul) =~ s|::|/|g;
     $file .= '.pm';
     if (-e "$Bin/../lib/$file" || -e "$Bin/../../lib/$file") {
          require $file;
          $modul->import();
          return 0;
     } else {
          return 1;
     }
}
sub remove_HTML {
     my $class = shift;
     my $text = shift;

     my $new_text = '';
     $text =~ s/<br>/\n/g;
     my $parser = HTML::TokeParser::Simple->new(string => $text);
     while (my $token = $parser->get_token) {
          $new_text .= $token->as_is if $token->is_text;
     }
#     $new_text =~ s/[^!-~]/ /g;
     return $new_text;
}
sub convert_ynof2bool {
     my $class = shift;
     my $item = lc(shift);

     return ($item eq "yes" || $item eq "on") ? eTRUE : eFALSE if (defined $item && $item ne "");
}
sub _build_member {
     my $synouser = shift;

     my ($rec,@raw_data2,@member,%member_);
     foreach $rec (@{$synouser}) {
          if (substr($rec,0,1) eq '@') {
               if (substr($rec,1) eq 'users') {
                    push @member, all_user();
               } else {
                    my $groupmembercommand = "/usr/syno/sbin/synogroup --get ".substr($rec,1)." | /usr/bin/tail -n +5 | /usr/bin/awk -F'[' '{ print \$2 }' | /usr/bin/awk -F']' '{ print \$1 }' 2> /dev/null|";
                    if (open(DAT, $groupmembercommand)) {
                         @raw_data2 = <DAT>;
                         close(DAT);
                    } else {
                         close(DAT);
                    }
                    if (@raw_data2) {
                         foreach my $groupmember_entry (@raw_data2) {
                              chomp($groupmember_entry);
                              push @member, $groupmember_entry;
                         }
                    }
               }
          } else {
               push @member, $rec;
          }
     }
     %member_ = map{$_,1}@member;
     return (keys %member_);
}
sub sendnotify {
     my $class = shift;
     my $user = shift;
     my $msg = shift;
     my $dsmbuild = shift;

     my $notifycommand = "/usr/syno/bin/synodsmnotify";
     if ($dsmbuild >= 1334) {
          my @notifyuser = split(",",$user);
          my $className = "SYNO.SDS._ThirdParty.App.ddnsupdater2";
          my $tagName = "err";
          my $result;
          if ($msg && $notifyuser[0] ne "") {
               my @groupmember = _build_member(\@notifyuser);
               foreach my $r (@groupmember) {
                    $r = encode_utf8($r);
                    $msg = encode_utf8($msg);
                    $msg =~ s/\n/<br>/g;
                    $result = `$notifycommand -c "$className" -t "$tagName" "$r" "DDNS Updater 2 Status report" "$msg"`;
               }
          }
     }
}
sub check_data_consistency {
     my $class = shift;
     my $configpath = shift;
     my $type = shift;

     my $duprovider = $class->read_config($configpath."/provider.json");
     my $missing = {'provider' => [], 'protocols' => [], 'warnings' => 0};
     my @grep_ret;
     if ($type eq "protocols" || $type eq "") {
          my $duprotocol = $class->read_config($configpath."/protocols.json");
          foreach my $prov (@{$duprovider->{'provider'}}) {
               @grep_ret = ();
               @grep_ret = grep {$_->{'name'} eq $prov->{'protocol'}} @{$duprotocol->{'protocols'}};
               push(@{$missing->{'protocols'}},"$prov->{'name'} -> $prov->{'protocol'}") if (!scalar(@grep_ret));
               $missing->{'warnings'}++ if (!scalar(@grep_ret));
          }
     }
     if ($type eq "provider" || $type eq "") {
          my $duconfig = $class->read_config($configpath."/config.json");
          foreach my $conf (@{$duconfig->{'config'}}) {
               @grep_ret = ();
               @grep_ret = grep {$_->{'name'} eq $conf->{'provider'}} @{$duprovider->{'provider'}};
               push(@{$missing->{'provider'}},"$conf->{'host'} -> $conf->{'provider'}") if (!scalar(@grep_ret));
               $missing->{'warnings'}++ if (!scalar(@grep_ret));
          }
     }
     return $missing;
}
1;