#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       Import.pm                                            #
#  Description: a perl package to convert DDNS Updater 1.xx config   #
#               to DDNS Updater 2.xx format                          #
#               or import DDNS Updater 2.xx data                     #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2015 by QTip                                         #
#  License:     BSD-style license (see LICENSE)                      #
#  ----------------------------------------------------------------  #
#  Version:     0.6 - 28/06/2015                                     #
#  for more information check the changelog                          #
#********************************************************************#
#  hosts with the following protocols need manual adjustment         #
#  after conversion of old config (probably host -> domain field):   #
#  cloudflare, duckdns, easydns, einsweb, gratisdns.dk, kontent,     #
#  namecheap, regfish, zoneedit                                      #
#********************************************************************#

package DDNSUpdater::Import;

use strict;
use warnings;

use base qw(Exporter);

our $VERSION     = '0.6';
our @EXPORT      = qw();
our @EXPORT_OK   = qw(import convert);

use Fcntl qw(:flock);
use File::Basename;
use DDNSUpdater::Functions;
use DDNSUpdater::CryptDecrypt;
#use Data::Dumper;

sub import {
     my $class = shift;
     my $configpath = shift;
     my $datafile = shift;
     my $clean = shift;
     my $upgrade = shift;
     my $json = shift;
     $datafile ||= '';
     $json ||= '';

     my ($dudata,$result,$ret,$new_data,$valid,@grep_ret,$nid);
     my %sum = ();
     $sum{'ctype'} = 2;
     $sum{'count'} = 0;
     $sum{'msg'} = "";

     if ($json eq "") {
          if (open(DAT,"<",$datafile)) {
               flock(DAT, LOCK_EX);
               $result = <DAT>;
               close(DAT);
          } else {
               close(DAT);
               $sum{'msg'} = "could not read file '$datafile";
               return (1,\%sum);
          }
     } else {
          $result = $json;
     }
     if ($result ne "" && $result =~ m/^({.*?})$/) {
          $new_data = JSON::XS->new->decode($result);
          $valid = _check_valid($new_data);
          if ($valid->{'root'}) {
               #import routine
               $sum{'file'} = $valid->{'root'};
               $sum{'count'} = scalar(@{$new_data->{$valid->{'root'}}});
               $sum{'imported'} = 0;
               if ($valid->{'root'} eq "options" || $valid->{'root'} eq "groups" || $clean == 1) {
                    $dudata = {$valid->{'root'} => []};
               } else {
                    $dudata = DDNSUpdater::Functions->read_config($configpath."/".$valid->{'root'}.".json");
               }
               foreach my $data (@{$new_data->{$valid->{'root'}}}) {
                    if ($valid->{'root'} eq "config") {
                         $nid = _get_next_id(\@{$dudata->{$valid->{'root'}}});
                         if (defined $nid) {
                              $data->{'id'} = "".$nid;
                              push(@{$dudata->{$valid->{'root'}}},$data);
                              $sum{'imported'}++;
                         } else {
                              $sum{'msg'} = "ERROR: ID not unique";
                              return (1,\%sum);
                         }
                    } else {
                         @grep_ret = grep {$_->{$valid->{'compare'}} eq $data->{$valid->{'compare'}}} @{$dudata->{$valid->{'root'}}};
#print $valid->{'root'}." = user\n" if (exists $grep_ret[0]->{'ftype'} && $grep_ret[0]->{'ftype'} eq "user");
                         if (scalar(@grep_ret)) {
                              if ($upgrade && exists $grep_ret[0]->{'ftype'} && ($grep_ret[0]->{'ftype'} eq $data->{'ftype'} || ($grep_ret[0]->{'ftype'} eq "addon" && $data->{'ftype'} eq "buildin"))) {
                                   map {if ($_->{$valid->{'compare'}} eq $data->{$valid->{'compare'}}) {$_ = $data}} @{$dudata->{$valid->{'root'}}};
                                   $sum{'imported'}++;
                              } else {
                                   next;
                              }
                         } else {
                              push(@{$dudata->{$valid->{'root'}}},$data);
                              $sum{'imported'}++;
                         }
                    }
               }
               $new_data = JSON::XS->new->encode($dudata);
               _write_config($configpath."/".$valid->{'root'}.".json",$new_data);
               return (0,\%sum);
          } else {
               # not a valid config 2.xx
               $sum{'file'} = ($json eq "" ? fileparse($datafile) : $datafile.' online update');
               $sum{'msg'} = "StatusImportNoDDU2Text";
               return (2,\%sum);
          }
     } else {
          $sum{'msg'} = "StatusImportNoDDU2Text";
          return (1,\%sum);
     }
}
sub convert {
     my $class = shift;
     my $configpath = shift;
     my $datafile = shift;
     my $clean = shift;

     my ($duprovider,$duconfig,$nid,$new_data);
     my @provider;
     my @oldconfig = ();
     my @confitem = ();
     my %sum = ();
     my $start = 0;
     my $c = 0;
     my ($im,$iv) = ('','');
     my $valid = 0;
     $sum{'file'} = 'ddclient';
     $sum{'ctype'} = 1;
     $sum{'count'} = 0;
     $sum{'imported'} = 0;
     $sum{'msg'} = "";

     $oldconfig[0] = {'host' => '', 'login' => '', 'password' => '', 'active' => eFALSE, 'group' => '0', 'provider' => '', 'domain' => '',
                       'type' => '', 'ssl' => eFALSE, 'ipv6' => eFALSE, 'mx' => '', 'mxpri' => '5', 'backmx' => eFALSE, 'static' => eFALSE,
                       'wildcard' => eFALSE, 'force_update' => 22, 'ip' => '', 'iptype' => 'auto', 'offline_shutdown' => eFALSE,
                       'usertext1' => '', 'usertext2' => '', 'usertext3' => '', 'useroption1' => '', 'offline' => eFALSE, 'script_update' => ''
     };
     $duprovider = DDNSUpdater::Functions->read_config($configpath."/provider.json");
     if ($clean == 1) {
          $duconfig = {'config' => []};
     } else {
          $duconfig = DDNSUpdater::Functions->read_config($configpath."/config.json");
     }
     if (-e $datafile) {
          if (open(HANDLE,"<",$datafile)) {
               while (<HANDLE>) {
                    # search for start of values
                    if ($start == 0 && index($_,"# DDNS Provider Parameters Section") != -1) {
                         $start = 1;
                         last if (!$valid);
                         next;
                    } elsif ($start == 0 && index($_,"##") != -1) {
#                         print "head: ".$_;
                         if (index($_,"ddclient.conf created") != -1) {
                              $valid = 1;
                         }
                    } elsif ($start == 0 && $_ ne "\x0A") {
#                         print "global: ".$_;
                    } elsif ($start == 1 && $_ eq "\x0A") {
                         $start = 2;
                         next;
                    }
                    if ($start == 2) {
                         if ($_ eq "\x0A") {
                              $c++;
                              $oldconfig[$c] = {'host' => '', 'login' => '', 'password' => '', 'active' => eFALSE, 'group' => '0', 'provider' => '', 'domain' => '',
                                                'type' => '', 'ssl' => eFALSE, 'ipv6' => eFALSE, 'mx' => '', 'mxpri' => '5', 'backmx' => eFALSE, 'static' => eFALSE,
                                                'wildcard' => eFALSE, 'force_update' => 22, 'ip' => '', 'iptype' => 'auto', 'offline_shutdown' => eFALSE,
                                                'usertext1' => '', 'usertext2' => '', 'usertext3' => '', 'useroption1' => '', 'offline' => eFALSE, 'script_update' => ''
                              };
                              next;
                         } else {
                              @confitem = ();
                              $_ =~ tr/\n//d;
#                              print "conf[".$c."]: ".$_."\n";
                              if (substr($_,0,4) eq "use=") {
                                   @confitem = split("=",$_);
                                   $im = DDNSUpdater::Functions->trim($confitem[1]);
                                   if (defined $im && $im eq "web") {
                                        $oldconfig[$c]->{'ip'} = '';
                                   } elsif (defined $im && substr($im,0,2) eq "ip") {
                                        $oldconfig[$c]->{'ip'} = DDNSUpdater::Functions->trim($confitem[2]);
                                   }
                              } else {
                                   @confitem = split("=",$_);
#                                   print Dumper(@confitem);
                                   if (!defined $confitem[1]) {
                                        $oldconfig[$c]->{'host'} = $confitem[0];
                                   } elsif ($confitem[0] eq "backupmx") {
                                        $oldconfig[$c]->{'backmx'} = DDNSUpdater::Functions->convert_ynof2bool($confitem[1]);
                                   } elsif ($confitem[0] eq "wildcard" || $confitem[0] eq "static") {
                                        $oldconfig[$c]->{$confitem[0]} = DDNSUpdater::Functions->convert_ynof2bool($confitem[1]);
                                   } elsif ($confitem[0] eq "server") {
                                        $confitem[1] = "cp.dnsmadeeasy.com" if ($confitem[1] eq "www.dnsmadeeasy.com");
                                        $confitem[1] = "control.dnspark.com" if ($confitem[1] eq "www.dnspark.com");
                                        $confitem[1] = "secure.dslreports.com" if ($confitem[1] eq "www.dslreports.com");
                                        $confitem[1] = "dip.gratisdns.de" if ($confitem[1] eq "www.gratisdns.de");
                                        $confitem[1] = "www.sitelutions.com" if ($confitem[1] eq "sitelutions.com");
                                        $confitem[1] = "update.spdns.de" if ($confitem[1] eq "www.spdns.de");
                                        $confitem[1] = "update.twodns.de" if ($confitem[1] eq "update.two-dns.de");
                                        @provider = grep {$_->{'server'} eq $confitem[1]} @{$duprovider->{'provider'}};
                                        if (scalar(@provider)) {
                                             $oldconfig[$c]->{'provider'} = $provider[0]->{'name'};
                                             $oldconfig[$c]->{'ssl'} = $provider[0]->{'ssl'};
                                             $oldconfig[$c]->{'force_update'} = $provider[0]->{'forceinterval'};
                                             delete $oldconfig[$c]->{'protocol'};
                                        }
                                   } elsif ($confitem[0] eq "password") {
                                        if (defined $confitem[1] && $confitem[1] ne "") {
                                             $oldconfig[$c]->{'password'} = DDNSUpdater::CryptDecrypt->encrypt_data($confitem[1],'','');
                                        }
                                   } elsif ($confitem[0] eq "custom") {
                                        next;
                                   } else {
                                        $oldconfig[$c]->{$confitem[0]} = $confitem[1];
                                   }
                              }
                         }
                    }
               }
          }
     }
     if ($valid && $oldconfig[0]->{'host'} ne "") {
          $sum{'count'} = scalar(@oldconfig);
          foreach my $oldcfg (@oldconfig) {
               $nid = undef;
               $nid = _get_next_id(\@{$duconfig->{'config'}});
               if (defined $nid) {
                    $oldcfg->{'id'} = "".$nid;
                    push (@{$duconfig->{'config'}},$oldcfg);
                    $sum{'imported'}++;
#print Dumper($oldcfg);
               } else {
                    $sum{'msg'} = "ERROR: ID not unique";
                    return (1,\%sum);
               }
          }
          $new_data = JSON::XS->new->encode($duconfig);
          _write_config($configpath."/config.json",$new_data);
          return (0,\%sum);
     } elsif ($valid && $oldconfig[0]->{'host'} eq "") {
          $sum{'count'} = 0;
          $sum{'msg'} = "StatusImportNoHostsText";
          return (1,\%sum);
     } else {
          $sum{'msg'} = "StatusImportNoDDU1Text";
          return (1,\%sum);
     }
}
sub _get_next_id {
     my @duconfig = @{(shift)};

     my (@records,$max_idx);
     my $i = 0;
     foreach my $key (@duconfig) {
         push(@records,int($key->{'id'}));
     }
     @records = sort { $a <=> $b } @records;
     $max_idx = scalar(@records) != 0 ? int($records[-1]) + 1 : 1;
     while (defined $records[$i] && $i <= $max_idx) {
          last if ($records[$i] != $i);
          $i++;
     }
     return ($i <= $max_idx) ? $i : undef;
}
sub _check_valid {
     my $data = shift;

     my $valid = ({'root' => '','compare' => ''});
     my @validtypes = ({'root' => 'sites','compare' => 'url'},{'root' => 'protocols','compare' => 'name'},{'root' => 'provider','compare' => 'name'},{'root' => 'responses','compare' => 'responsename'},
                      {'root' => 'config','compare' => 'host'},{'root' => 'options','compare' => ''},{'root' => 'groups','compare' => 'name'});
     foreach my $t (@validtypes) {
          if (exists $data->{$t->{'root'}}) {
               $valid = $t;
               last;
          }
     }
     return $valid;
}
sub _write_config {
     my $datafile = shift;
     my $value = shift;

     if (open(DATA,">",$datafile)) {
          flock(DATA, LOCK_EX);
          print DATA $value;
          close (DATA);
          return 0;
     } else {
          close(DATA);
          return 1;
     }
}
1;