#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       Logger.pm                                            #
#  Description: misc perl functions                                  #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2014-2015 by QTip                                    #
#  License:     BSD-style license (see LICENSE)                      #
#  ----------------------------------------------------------------  #
#  Version:     0.45 - 13/12/2015                                    #
#  for more information check the changelog                          #
#********************************************************************#

package DDNSUpdater::Logger;

use strict;
use warnings;

use base qw(Exporter);

our $VERSION     = '0.45';
our @EXPORT      = qw(info warning error critical debug noconnect success failed);
our @EXPORT_OK   = ();

use POSIX qw(strftime);
use Sys::Syslog qw(:DEFAULT);
use Log::Tiny;

my $duhome = '/var/packages/ddnsupdater2/target';
unless (-d $duhome."/log") {mkdir($duhome."/log",0777) || die "Could not create debug log dir ($!)\n\n";}
my $debuglogfile = $duhome."/log/ddud_debug.log";
our $debuglog = Log::Tiny->new($debuglogfile) || die "Could not create logfile! (".Log::Tiny->errstr.")\n\n";

my @anonymizetags = ('login','user','username','email','password','pass','pw','auth','apikey','key','tkn','token','u','p');
my %anonymizeitems = map {$_ => 1} @anonymizetags;

sub info {_log_message('INFO','info',@_);}
sub warning {_log_message('WARNING','info',@_);}
sub error {_log_message('ERROR','info',@_);}
sub critical {_log_message('CRITICAL','info',@_);}
sub debug {_log_message('DEBUG','debug',@_);}
sub noconnect {_log_message('NOCONNECT','info',@_);}
sub success {_log_message('SUCCESS','info',@_);}
sub failed {_log_message('FAILED','info',@_);}

sub _log_message {
     my $priotext = shift;
     my $prio = shift;
     my $message = shift;
     my $anonym = shift;
     $anonym ||= 0;

     # we need to do this so that the time stamp is updated
     $debuglog->format(strftime("%Y-%m-%d %H:%M:%S", localtime)."\t%c\t: %m\n");

     if (ref($message) eq "HASH" || ref($message) eq "ARRAY") {
          _object2message($priotext,$prio,$message,0,$anonym);
     } else {
          $priotext .= (" " x (11 - length($priotext)));
          if ($prio eq "debug") {
               $message = _anonymize_ip($message) if (defined $message && $anonym);
               $debuglog->DEBUG($message);
          } else {
               openlog('ddud','','daemon');
               syslog($prio,$priotext.": ".$message);
               closelog();
          }
     }
}
sub _object2message {
     my $priotext = shift;
     my $prio = shift;
     my $object = shift;
     my $inner = shift;
     my $anonym = shift;
     $anonym ||= 0;

     # we need to do this so that the time stamp is updated
     $debuglog->format(strftime("%Y-%m-%d %H:%M:%S", localtime)."\t%c\t: %m\n");

     $priotext .= (" " x (11 - length($priotext)));

     while (my ($key, $value) = each(%$object)) {
          next if ($anonym && exists($anonymizeitems{$key}));
          if (ref($value) eq "HASH") {
               if ($prio eq "debug") {
                    $debuglog->DEBUG($key.": ");
               } else {
                    openlog('ddud','','daemon');
                    syslog($prio,$priotext.": ".$key.": ");
                    closelog();
               }
               _object2message($priotext,$prio,$value,1,$anonym);
          } else {
               $value = join(", ",@{$value}) if (ref($value) eq "ARRAY");
               if ($prio eq "debug") {
                    $value = _anonymize_ip($value) if (defined $value && $anonym);
                    $debuglog->DEBUG(($inner?"\t":"").$key.": ".(defined $value?$value:'n/a'));
               } else {
                    openlog('ddud','','daemon');
                    syslog($prio,$priotext.": ".($inner?"\t":"").$key.": ".(defined $value?$value:'n/a'));
                    closelog();
               }
          }
     }
}
sub _anonymize_ip {
     my $value = shift;

     $value =~ s/(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/x.x.x.x/g;
     $value =~ s/(\w{1,4}|0)((\:|\:\:)(\w{1,4}|0)){3,7}/x:x:x:x:x:x:x:x/gi;
     return $value;
}
1;
