#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       Module::example <== update with desired package name #
#  Description: default module to update ddns hosts                  #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2014-2016 by QTip                                    #
#  License:     BSD-style license (see LICENSE)                      #
#  ----------------------------------------------------------------  #
#  Version:     0.36 - 05/05/2016                                    #
#  for more information check the changelog                          #
#********************************************************************#

package DDNSUpdater::Module::example; # <== update with desired package name

# To integrate and use this module for testing, read the following steps:
# 1. put the module name into /var/packages/ddnsupdater2/etc/modules.json, according the other modules e.g:
#    {"modules":["goip","zoneedit1","regfish","kontent","gnudip_http","freedns","enom","dnsexit","default","cloudflare2","zonomi","pleskdyndns","example"]} <== your module name
# 2. run DDNS Updater gui and access protocol management by clicking on 'Settings & configuration', select tab 'Protocol management'
# 3. create a new protocol and select your module
# 4. fill all fields of the form and grid with help of the providers protocol api, save new protocol
# 5. go to provider management and create a new provider
# 6. fill all fields of the form with help of the providers protocol api, save new provider
# 7. create new host entry and select your new provider/protocol
# 8. test your protocol
# Ready ;)

use strict;
use warnings;

use base qw(Exporter);

our $VERSION     = '0.36';             # <== update with desired package version
our @EXPORT      = ();
our @EXPORT_OK   = qw(update_protocol);

use DDNSUpdater::Logger;
use if $ENV{'DDU_DEBUG'}, "Data::Dumper";

my $EOL = $::EOL;
my $debuglog_anonym = $::debuglog_anonym;

sub update_protocol {
     my $class  = shift;
     my $updatevars  = shift;
     my $uh = shift;

     # determine whether use url or http auth
     my ($error,$errortext,$ret,$pw);
     if (exists $updatevars->{'auth'}->{'password'} && defined $updatevars->{'auth'}->{'password'}) {
          $pw = DDNSUpdater::CryptDecrypt->decrypt_data($updatevars->{'auth'}->{'password'},'','');
          $updatevars->{'auth'}->{'password'} = $pw if ($pw ne "");
     } else {
          $pw = DDNSUpdater::CryptDecrypt->decrypt_data($updatevars->{'tags'}->{$updatevars->{'passtag'}},'','');
          $updatevars->{'tags'}->{$updatevars->{'passtag'}} = $pw if ($pw ne "");
     }
     $pw = '';

     # canonify url
     my $postdata;
     my $updateurl = DDNSUpdater::Tools->canonify_url($updatevars);
     $postdata = ($updatevars->{'req_method'} eq "POST" ? $updatevars->{'tags'} : {});

     # start update process
     if ($updateurl ne "") {
          my $updateurl_ = $updateurl;
          # anonymizing
          $updateurl_ =~ s/($updatevars->{'logintag'}|$updatevars->{'passtag'})=([^&]*)/$1=xxx/g if ($debuglog_anonym);
          print $updateurl_.$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug($updateurl_,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($updateurl,$updatevars->{'ipv6'},$updatevars->{'ssl'},$updatevars->{'auth'},$updatevars->{'req_method'},$postdata);

          $ret = DDNSUpdater::Functions->trim($ret);
          my $dret = $ret;
          $dret =~ s/\n/, /g;
          debug("return: '".$dret."'",$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "return: '".$dret."'".$EOL if $ENV{'DDU_DEBUG'};
          $ret = DDNSUpdater::Functions->remove_HTML($ret);
          # delete unwanted chars in return string
          $ret =~ s/[\x00-\x09\x0B\x0C\x0E-\x1F\x22-\x27\x2A]//ig;
          # evaluating response (protocol specific code)
          # return scheme must be $error,$errortext,$codeHash
          # $error:     error number
          # $errortext: error description
          # $codeHash:  the prepared unique return code hash, hash with errortext as code if error != 0
          #             format:
          #                   {
          #                     'host id(0)' => {
          #                        'host' => 'xxx.xxx.xxx.xx',
          #                        'code' => 'return/error-code'
          #                     },
          #                     'host id(x)' => {
          #                        'host' => 'xxx.xxx.xxx.xx',
          #                        'code' => 'return/error-code'
          #                     }
          #                   }
          if ($error == 0) {
               my $max_hosts = scalar(@{$uh});
               my $currenthost;
               my (@ret,$code);
               @ret = split(/\n/,$ret);
               foreach my $line (@ret) {
                    $currenthost = shift @{$uh};
                    my $code = lc($line);
                    $result{$currenthost->{'id'}}{'host'} = $currenthost->{'host'};
                    $result{$currenthost->{'id'}}{'code'} = (defined $code ? encode_utf8(DDNSUpdater::Functions->trim(lc($code))) : '');
               }
               map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                    $result{$_->{'id'}}{'code'} = $result{$currenthost->{'id'}}{'code'};
               } @{$uh} if ($max_hosts > scalar(keys %result));
               return ($error,$errortext,%result);
          } else {
               map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                    $result{$_->{'id'}}{'code'} = $errortext;
               } @{$uh};
               return ($error,$errortext,%result);
          }
     } else {
          map {$result{$_->{'id'}}{'host'} = $_->{'host'};
               $result{$_->{'id'}}{'code'} = 'int: Could not build update url';
          } @{$uh};
          return (1000,'int: Could not build update url',%result);
     }
}
1;