#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       Module::cloudflare2                                  #
#  Description: module to update cloudflare.com ddns hosts           #
#               uses cloudflare v2 api                               #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2014-2016 by QTip                                    #
#               adopted from code by Peter Roberts 2013-9-26,        #
#                                      2014-6-22: blog.peter-r.co.uk #
#  License:     BSD-style license (see LICENSE)                      #
#  ----------------------------------------------------------------  #
#  Version:     0.46 - 05/05/2016                                    #
#  for more information check the changelog                          #
#********************************************************************#

package DDNSUpdater::Module::cloudflare2;

use strict;
use warnings;

use base qw(Exporter);

our $VERSION     = '0.46';
our @EXPORT      = ();
our @EXPORT_OK   = qw(update_protocol);

use DDNSUpdater::Logger;
use if $ENV{'DDU_DEBUG'}, "Data::Dumper";

my $EOL = $::EOL;
my $debuglog_anonym = $::debuglog_anonym;

sub update_protocol {
     my $class  = shift;
     my $updatevars  = shift;
     my $uh = shift;

     # determine whether use url or http auth
     my ($error,$errortext,$ret,$pw);
     my %result = ();
     if (exists $updatevars->{'auth'}->{'password'} && defined $updatevars->{'auth'}->{'password'}) {
          $pw = DDNSUpdater::CryptDecrypt->decrypt_data($updatevars->{'auth'}->{'password'},'','');
          $updatevars->{'auth'}->{'password'} = $pw if ($pw ne "");
     } else {
          $pw = DDNSUpdater::CryptDecrypt->decrypt_data($updatevars->{'tags'}->{$updatevars->{'passtag'}},'','');
          $updatevars->{'tags'}->{$updatevars->{'passtag'}} = $pw if ($pw ne "");
     }
     $pw = '';

     # canonify url
     my $wantedip = $updatevars->{'tags'}->{'content'};
     my $wantedname = $updatevars->{'tags'}->{'name'};
     $updatevars->{'static_params'} = 'a=rec_load_all';
     delete $updatevars->{'tags'}->{'name'}; delete $updatevars->{'tags'}->{'content'};
     my $postdata;
     my $updateurl = DDNSUpdater::Tools->canonify_url($updatevars);
     $postdata = ($updatevars->{'req_method'} eq "POST" ? $updatevars->{'tags'} : {});

     # start update process
     if ($updateurl ne "") {
          my ($code,$response);
          my $updateurl_ = $updateurl;
          $updateurl_ =~ s/($updatevars->{'logintag'}|$updatevars->{'passtag'})=([^&]*)/$1=xxx/g if ($debuglog_anonym);
          print $updateurl_.$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug($updateurl_,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "get domain id".$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug("get domain id") if ($ENV{'DDU_DEBUG_LOG'});
          ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($updateurl,$updatevars->{'ipv6'},$updatevars->{'ssl'},$updatevars->{'auth'},$updatevars->{'req_method'},$postdata);
          $ret = DDNSUpdater::Functions->trim($ret);
          debug("return: '".$ret."'",$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "return: '".$ret."'".$EOL if $ENV{'DDU_DEBUG'};
          if ($error == 0 && $ret =~ m/^({.*?})$/) {
               $ret =~ s/^.*?\n\n//s;
               $response = JSON::XS->new->decode($ret);
               if ($response->{'result'} eq "error") {
                    $code = (exists $response->{'err_code'} && $response->{'err_code'} ne "") ? lc($response->{'err_code'}) : lc($response->{'msg'});
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = $code} @{$uh};
                    return ($error,$errortext,%result);
               }
               # get domain id from json
               my ($id) = map {$_->{'name'} eq $wantedname ? $_->{'rec_id'} : ()} @{$response->{'response'}->{'recs'}->{'objs'}};
               if ($id) {
                    $updatevars->{'tags'}->{'id'} = $id;
               } else {
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = 'no domain id found'} @{$uh};
                    return ($error,$errortext,%result);
               }
          } else {
               map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                    $result{$_->{'id'}}{'code'} = $errortext} @{$uh};
               return ($error,$errortext,%result);
          }

          # canonify url with id
          $updatevars->{'tags'}->{'content'} = $wantedip;
          $updatevars->{'tags'}->{'name'} = $wantedname;
          $updatevars->{'static_params'} = 'a=rec_edit';
          my $postdata;
          my $updateurl = DDNSUpdater::Tools->canonify_url($updatevars);
          $postdata = ($updatevars->{'req_method'} eq "POST" ? $updatevars->{'tags'} : {});
          $updateurl_ = $updateurl;
          $updateurl_ =~ s/($updatevars->{'logintag'}|$updatevars->{'passtag'})=([^&]*)/$1=xxx/g if ($debuglog_anonym);
          print $updateurl_.$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug($updateurl_,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "update ip of domain".$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug("update ip of domain") if ($ENV{'DDU_DEBUG_LOG'});
          ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($updateurl,$updatevars->{'ipv6'},$updatevars->{'ssl'},$updatevars->{'auth'},$updatevars->{'req_method'},$postdata);
          $ret = DDNSUpdater::Functions->trim($ret);
          debug("return: '".$ret."'",$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "return: '".$ret."'".$EOL if $ENV{'DDU_DEBUG'};

          # evaluating response (protocol specific code)
          # return scheme must be $error,$errortext,$codeHash
          # $error:     error number
          # $errortext: error description
          # $codeHash:  the prepared unique return code hash, hash with errortext as code if error != 0
          #             format:
          #                   {
          #                     'host id(0)' => {
          #                        'host' => 'xxx.xxx.xxx.xx',
          #                        'code' => 'return/error-code'
          #                     }
          #                   }

          if ($error == 0 && $ret =~ m/^({.*?})$/) {
               $ret =~ s/^.*?\n\n//s;
               $response = JSON::XS->new->decode($ret);
               if ($response->{'result'} eq "error") {
                    $code = (exists $response->{'err_code'} && $response->{'err_code'} ne "") ? lc($response->{'err_code'}) : lc($response->{'msg'});
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = $code} @{$uh};
                    return ($error,$errortext,%result);
               } else {
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = 'ok'} @{$uh};
                    return ($error,$errortext,%result);
               }
          } else {
               map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                    $result{$_->{'id'}}{'code'} = $errortext} @{$uh};
               return ($error,$errortext,%result);
          }
     } else {
          map {$result{$_->{'id'}}{'host'} = $_->{'host'};
               $result{$_->{'id'}}{'code'} = 'int: Could not build update url'} @{$uh};
          return (1000,'int: Could not build update url',%result);
     }
}
1;