#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       Module::freedns                                      #
#  Description: module to update freedns.afraid.org ddns hosts       #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2014-2016 by QTip                                    #
#  License:     BSD-style license (see LICENSE)                      #
#  ----------------------------------------------------------------  #
#  Version:     0.42 - 03/07/2016                                    #
#  for more information check the changelog                          #
#********************************************************************#

package DDNSUpdater::Module::freedns;

use strict;
use warnings;

use base qw(Exporter);

our $VERSION     = '0.42';
our @EXPORT      = ();
our @EXPORT_OK   = qw(update_protocol);

use DDNSUpdater::Logger;
use Digest::SHA1 qw(sha1_hex);
use if $ENV{'DDU_DEBUG'}, "Data::Dumper";

my $EOL = $::EOL;
my $debuglog_anonym = $::debuglog_anonym;

sub update_protocol {
     my $class  = shift;
     my $updatevars  = shift;
     my $uh = shift;

     # determine whether use url or http auth
     my ($error,$errortext,$ret,$pw);
     my %result = ();
     if (exists $updatevars->{'auth'}->{'password'} && defined $updatevars->{'auth'}->{'password'}) {
          $pw = DDNSUpdater::CryptDecrypt->decrypt_data($updatevars->{'auth'}->{'password'},'','');
          $updatevars->{'auth'}->{'password'} = $pw if ($pw ne "");
     } else {
          $pw = DDNSUpdater::CryptDecrypt->decrypt_data($updatevars->{'tags'}->{$updatevars->{'passtag'}},'','');
          $updatevars->{'tags'}->{$updatevars->{'passtag'}} = $pw if ($pw ne "");
     }
     $pw = '';

     # canonify url
     # post request method not available
     $updatevars->{'static_params'} = 'action=getdyndns';
     $updatevars->{'tags'}->{'sha'} = sha1_hex("$updatevars->{'auth'}->{'login'}|$updatevars->{'auth'}->{'password'}");
     my $wantedip = $updatevars->{'tags'}->{'address'};
     my $wantedhost = $updatevars->{'tags'}->{'host'};
     delete $updatevars->{'tags'}->{'address'}; delete $updatevars->{'tags'}->{'host'};
     my $updateurl = DDNSUpdater::Tools->canonify_url($updatevars);

     # start update process
     if ($updateurl ne "") {
          my $updateurl_ = $updateurl;
          $updateurl_ =~ s/($updatevars->{'logintag'}|$updatevars->{'passtag'})=([^&]*)/$1=xxx/g if ($debuglog_anonym);
          print $updateurl_.$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug($updateurl_,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "get domain urls".$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug("get domain urls") if ($ENV{'DDU_DEBUG_LOG'});
          ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($updateurl,$updatevars->{'ipv6'},$updatevars->{'ssl'},$updatevars->{'auth'},$updatevars->{'req_method'},{});
          $ret = DDNSUpdater::Functions->trim($ret);
          debug("return: '".$ret."'",$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "return: '".$ret."'".$EOL.$EOL if $ENV{'DDU_DEBUG'};

          # evaluating response (protocol specific code)
          # return scheme must be $error,$errortext,$codeHash
          # $error:     error number
          # $errortext: error description
          # $codeHash:  the prepared unique return code hash, hash with errortext as code if error != 0
          #             format:
          #                   {
          #                     'host id(0)' => {
          #                        'host' => 'xxx.xxx.xxx.xx',
          #                        'code' => 'return/error-code'
          #                     }
          #                   }

          if ($error == 0) {
               my $code;
               my @response = split(":",$ret);
               if (@response && $response[0] eq 'error') {
                    $code = $response[1];
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = $code} @{$uh};
                    return ($error,$errortext,%result);
               }
               my @lines = split("\n", $ret);
               my @entry;
               $updateurl = '';
               foreach my $l (@lines) {
                    @entry = split(/\|/, $l);
                    $updateurl = $entry[2] if (exists($entry[2]) && $entry[0] eq $wantedhost);
               }
               if (!$updateurl) {
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = "Could not get freedns update url for host '".$wantedhost."'"} @{$uh};
                    return (1000,"Could not get freedns update url for host '".$wantedhost."'",%result);
               }
               $updatevars->{'auth'}->{'login'} = 'guest';
               $updatevars->{'auth'}->{'password'} = 'guest';
               $updateurl .= '&address='.$wantedip;
               my $updateurl_ = $updateurl;
               $updateurl_ =~ s/($updatevars->{'logintag'}|$updatevars->{'passtag'})=([^&]*)/$1=xxx/g if ($debuglog_anonym);
               print $updateurl_.$EOL.$EOL if $ENV{'DDU_DEBUG'};
               debug($updateurl_,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});

               print "update ip of domain".$EOL.$EOL if $ENV{'DDU_DEBUG'};
               debug("update ip of domain") if ($ENV{'DDU_DEBUG_LOG'});
               ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($updateurl,$updatevars->{'ipv6'},$updatevars->{'ssl'},$updatevars->{'auth'},$updatevars->{'req_method'},{});
               $ret = DDNSUpdater::Functions->trim(lc($ret));
               debug("return: '".$ret."'",$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
               print "return: '".$ret."'".$EOL if $ENV{'DDU_DEBUG'};

               # evaluating response (protocol specific code)
               # return scheme must be $error,$errortext,$codeHash
               # $error:     error number
               # $errortext: error description
               # $codeHash:  the prepared unique return code hash, hash with errortext as code if error != 0
               #             format:
               #                   {
               #                     'host id(0)' => {
               #                        'host' => 'xxx.xxx.xxx.xx',
               #                        'code' => 'return/error-code'
               #                     }
               #                   }

               if ($error == 0) {
                    if ($ret =~ /updated.*$wantedhost.*to.*$wantedip/) {
                         $code = 'ok';
                    } else {
                         @response = split(":",$ret);
                         if (@response && $response[1] =~ /address.*has not changed/) {
                              $code = 'nochg';
                         } else {
                              $code = DDNSUpdater::Functions->trim(lc($response[1]));
                         }
                    }
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = $code} @{$uh};
                    return ($error,$errortext,%result);
               } else {
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = $errortext} @{$uh};
                    return ($error,$errortext,%result);
               }
          } else {
               map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                    $result{$_->{'id'}}{'code'} = $errortext} @{$uh};
               return ($error,$errortext,%result);
          }
     } else {
          map {$result{$_->{'id'}}{'host'} = $_->{'host'};
               $result{$_->{'id'}}{'code'} = 'int: Could not build update url'} @{$uh};
          return (1000,'int: Could not build update url',%result);
     }
}
1;