#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       Module::gnudip_http                                  #
#  Description: module to update gnudip ddns hosts                   #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2014-2016 by QTip                                    #
#  License:     BSD-style license (see LICENSE)                      #
#  ----------------------------------------------------------------  #
#  Version:     0.31 - 05/05/2016                                    #
#  for more information check the changelog                          #
#********************************************************************#

package DDNSUpdater::Module::gnudip_http;

use strict;
use warnings;

use base qw(Exporter);

our $VERSION     = '0.31';
our @EXPORT      = ();
our @EXPORT_OK   = qw(update_protocol);

use DDNSUpdater::Logger;
use Digest::MD5 qw(md5_hex);
use if $ENV{'DDU_DEBUG'}, "Data::Dumper";

my $EOL = $::EOL;
my $debuglog_anonym = $::debuglog_anonym;

sub update_protocol {
     my $class  = shift;
     my $updatevars  = shift;
     my $uh = shift;

     my ($error,$errortext,$ret,$pw);
     my %result = ();
     $pw = DDNSUpdater::CryptDecrypt->decrypt_data($updatevars->{'tags'}->{$updatevars->{'passtag'}},'','');
     $updatevars->{'tags'}->{$updatevars->{'passtag'}} = md5_hex($pw) if ($pw ne "");
     $pw = '';

     my %errorcodes = (
          '0' => 'ok', '1' => 'noauth', '2' => 'offline'
     );

     my $updateurl = $updatevars->{'server'}.$updatevars->{'script'};

     # start update process
     if ($updateurl ne "") {
          my $updateurl_ = $updateurl;
          $updateurl_ =~ s/($updatevars->{'logintag'}|$updatevars->{'passtag'})=([^&]*)/$1=xxx/g if ($debuglog_anonym);
          print $updateurl_.$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug($updateurl_,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "get salt".$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug("get salt") if ($ENV{'DDU_DEBUG_LOG'});
          ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($updateurl,$updatevars->{'ipv6'},$updatevars->{'ssl'},$updatevars->{'auth'},$updatevars->{'req_method'},{});
          $ret = DDNSUpdater::Functions->trim($ret);
          debug("return: '".$ret."'",$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "return: '".$ret."'".$EOL.$EOL if $ENV{'DDU_DEBUG'};

          # evaluating response (protocol specific code)
          # return scheme must be $error,$errortext,$codeHash
          # $error:     error number
          # $errortext: error description
          # $codeHash:  the prepared unique return code hash, hash with errortext as code if error != 0
          #             format:
          #                   {
          #                     'host id(0)' => {
          #                        'host' => 'xxx.xxx.xxx.xx',
          #                        'code' => 'return/error-code'
          #                     }
          #                   }

          if ($error == 0) {
               my $code;
               my @lines = split("\n", $ret);
               my %secure = ();
               $updateurl = '';
               foreach my $l (@lines) {
                    if ($l =~ /^<meta name=\"(.*)\" content=\"(.*)\">$/) {
                         $secure{$1} = $2;
                    }
               }
               if (!$secure{'salt'}) {
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = "Server did not send salt for '".$updatevars->{'tags'}->{$updatevars->{'logintag'}}." ".$updatevars->{'tags'}->{'domn'}."'"} @{$uh};
                    return (1000,"Server did not send salt for '".$updatevars->{'tags'}->{$updatevars->{'logintag'}}." ".$updatevars->{'tags'}->{'domn'}."'",%result);
               }

               # canonify url
               $updatevars->{'static_params'} = 'reqc=0';
               $updatevars->{'tags'}->{'salt'} = $secure{'salt'};
               $updatevars->{'tags'}->{'time'} = $secure{'time'};
               $updatevars->{'tags'}->{'sign'} = $secure{'sign'};
               $updatevars->{'tags'}->{$updatevars->{'passtag'}} = md5_hex("$updatevars->{'tags'}->{$updatevars->{'passtag'}}.$secure{'salt'}");
               my $postdata;
               my $updateurl = DDNSUpdater::Tools->canonify_url($updatevars);
               $postdata = ($updatevars->{'req_method'} eq "POST" ? $updatevars->{'tags'} : {});
               $updateurl_ = $updateurl;
               $updateurl_ =~ s/($updatevars->{'logintag'}|$updatevars->{'passtag'})=([^&]*)/$1=xxx/g if ($debuglog_anonym);
               print $updateurl_.$EOL.$EOL if $ENV{'DDU_DEBUG'};
               debug($updateurl_,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});

               print "update ip of domain".$EOL.$EOL if $ENV{'DDU_DEBUG'};
               debug("update ip of domain") if ($ENV{'DDU_DEBUG_LOG'});
               ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($updateurl,$updatevars->{'ipv6'},$updatevars->{'ssl'},$updatevars->{'auth'},$updatevars->{'req_method'},$postdata);
               $ret = DDNSUpdater::Functions->trim(lc($ret));
               debug("return: '".$ret."'",$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
               print "return: '".$ret."'".$EOL if $ENV{'DDU_DEBUG'};

               # evaluating response (protocol specific code)
               # return scheme must be $error,$errortext,$codeHash
               # $error:     error number
               # $errortext: error description
               # $codeHash:  the prepared unique return code hash, hash with errortext as code if error != 0
               #             format:
               #                   {
               #                     'host id(0)' => {
               #                        'host' => 'xxx.xxx.xxx.xx',
               #                        'code' => 'return/error-code'
               #                     }
               #                   }

               if ($error == 0) {
                    my @lines = split("\n", $ret);
                    my %datas = ();
                    $updateurl = '';
                    foreach my $l (@lines) {
                         if ($l =~ /^<meta name=\"(.*)\" content=\"(.*)\">$/) {
                              $datas{$1} = $2;
                         }
                    }
                    if (!defined $datas{'retc'}) {
                         map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                              $result{$_->{'id'}}{'code'} = "Server did not respond to login '".$updatevars->{'tags'}->{$updatevars->{'logintag'}}." ".$updatevars->{'tags'}->{'domn'}."'"} @{$uh};
                         return (1000,"Server did not respond to login '".$updatevars->{'tags'}->{$updatevars->{'logintag'}}." ".$updatevars->{'tags'}->{'domn'}."'",%result);
                    }
                    my $code = ($datas{'retc'} ne "" && exists $errorcodes{$datas{'retc'}}) ? $errorcodes{$datas{'retc'}} : "Server did not respond to login attempt for '".$updatevars->{'tags'}->{$updatevars->{'logintag'}}." ".$updatevars->{'tags'}->{'domn'}."'";
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = $code} @{$uh};
                    return ($error,$errortext,%result);
               } else {
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = $errortext} @{$uh};
                    return ($error,$errortext,%result);
               }
          } else {
               map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                    $result{$_->{'id'}}{'code'} = $errortext} @{$uh};
               return ($error,$errortext,%result);
          }
     } else {
          map {$result{$_->{'id'}}{'host'} = $_->{'host'};
               $result{$_->{'id'}}{'code'} = 'int: Could not build update url'} @{$uh};
          return (1000,'int: Could not build update url',%result);
     }
}
1;