#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       Module::inwx                                         #
#  Description: module to update inwx ddns hosts,                    #
#               can possibly be used for other xml rpc server        #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2014-2016 by QTip                                    #
#  License:     BSD-style license (see LICENSE)                      #
#  ----------------------------------------------------------------  #
#  Version:     0.1 - 05/05/2016                                     #
#  for more information check the changelog                          #
#********************************************************************#

package DDNSUpdater::Module::inwx;

use strict;
use warnings;

use base qw(Exporter);

our $VERSION     = '0.1';
our @EXPORT      = ();
our @EXPORT_OK   = qw(update_protocol);

use Scalar::Util qw(looks_like_number);
use XML::Mini::Document;
use DDNSUpdater::Logger;
use if $ENV{'DDU_DEBUG'}, "Data::Dumper";

my $EOL = $::EOL;
my $debuglog_anonym = $::debuglog_anonym;

sub update_protocol {
     my $class  = shift;
     my $updatevars  = shift;
     my $uh = shift;

     # determine whether use url or http auth
     my ($error,$errortext,$ret,$pw);
     my %result = ();
     $pw = DDNSUpdater::CryptDecrypt->decrypt_data($updatevars->{'tags'}->{$updatevars->{'passtag'}},'','');
     $updatevars->{'tags'}->{$updatevars->{'passtag'}} = $pw if ($pw ne "");
     $pw = '';

     # canonify url
     my $updateurl = DDNSUpdater::Tools->canonify_url($updatevars);

     # start update process
     if ($updateurl ne "") {
          my $updateurl_ = $updateurl;
          $updateurl_ =~ s/($updatevars->{'logintag'}|$updatevars->{'passtag'})=([^&]*)/$1=xxx/g if ($debuglog_anonym);
          print $updateurl_.$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug($updateurl_,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});

          # request domain record id
          # disassemble domain
          my @domainnameparts = split(/\./,$updatevars->{'tags'}->{'domain'});
          my $domain = join(".",@domainnameparts[-2..-1]);
          my $hostname = join(".",@domainnameparts[0..$#domainnameparts-2]);

          my $postdata = xmlrpc_encode_request('nameserver.info',{'user' => $updatevars->{'tags'}->{$updatevars->{'logintag'}}, 'pass' => $updatevars->{'tags'}->{$updatevars->{'passtag'}},'domain' => $domain, 'name' => $hostname, 'type' => $updatevars->{'tags'}->{'type'}});
          my $postdata_ = $postdata;
          $postdata_ =~ s/(<name>(user|pass)<\/name><value><\w+>)([^<]*)/$1xxx/g if ($debuglog_anonym);
          print "get domain record id".$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug("get domain record id") if ($ENV{'DDU_DEBUG_LOG'});
          print $postdata_.$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug($postdata_,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($updateurl,$updatevars->{'ipv6'},$updatevars->{'ssl'},{},$updatevars->{'req_method'},{'--xmlrpc'=>$postdata});
          $ret = DDNSUpdater::Functions->trim($ret);
          $ret =~ s/\n/ /g;
          debug("return: '".$ret."'",$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "return: '".$ret."'".$EOL if $ENV{'DDU_DEBUG'};

          if ($error == 0) {
               my ($code,$xml_retcode);
               my $retXML = XML::Mini::Document->new();
               $retXML->parse($ret);
               my $retHash = $retXML->toHash();
               if (exists $retHash->{'methodResponse'}->{'fault'}) {
                    $xml_retcode = xmlrpc_get_fault($retHash,'faultCode');
                    $code = lc(xmlrpc_get_fault($retHash,'faultString'));
               } else {
                    $xml_retcode = xmlrpc_get_code($retHash,'code');
                    $code = lc(xmlrpc_get_code($retHash,'msg'));
               }
               if (defined $xml_retcode && $xml_retcode != 1000) {
                    map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                         $result{$_->{'id'}}{'code'} = $code} @{$uh};
                    return ($error,$errortext,%result);
               } else {
                    # retrieve record id of domain from xml
                    my $recordid = xmlrpc_get_recordid($retHash);
                    if ($recordid) {
                         $updatevars->{'tags'}->{'id'} = $recordid;
                    } else {
                         map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                              $result{$_->{'id'}}{'code'} = 'no domain id found'} @{$uh};
                         return ($error,$errortext,%result);
                    }
               }
          } else {
               map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                    $result{$_->{'id'}}{'code'} = $errortext} @{$uh};
               return ($error,$errortext,%result);
          }

          # update domain record
          $postdata = xmlrpc_encode_request('nameserver.updateRecord',{'user' => $updatevars->{'tags'}->{$updatevars->{'logintag'}}, 'pass' => $updatevars->{'tags'}->{$updatevars->{'passtag'}},'id' => $updatevars->{'tags'}->{'id'}, 'content' => $updatevars->{'tags'}->{'ip'}});
          $postdata_ = $postdata;
          $postdata_ =~ s/(<name>(user|pass)<\/name><value><\w+>)([^<]*)/$1xxx/g if ($debuglog_anonym);
          print "update domain record".$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug("update domain record") if ($ENV{'DDU_DEBUG_LOG'});
          print $postdata_.$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug($postdata_,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($updateurl,$updatevars->{'ipv6'},$updatevars->{'ssl'},{},$updatevars->{'req_method'},{'--xmlrpc'=>$postdata});
          $ret = DDNSUpdater::Functions->trim($ret);
          $ret =~ s/\n/ /g;
          debug("return: '".$ret."'",$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "return: '".$ret."'".$EOL if $ENV{'DDU_DEBUG'};

          # evaluating response (protocol specific code)
          # return scheme must be $error,$errortext,$codeHash
          # $error:     error number
          # $errortext: error description
          # $codeHash:  the prepared unique return code hash, hash with errortext as code if error != 0
          #             format:
          #                   {
          #                     'host id(0)' => {
          #                        'host' => 'xxx.xxx.xxx.xx',
          #                        'code' => 'return/error-code'
          #                     }
          #                   }

          if ($error == 0) {
               my ($code,$xml_retcode);
               my $retXML = XML::Mini::Document->new();
               $retXML->parse($ret);
               my $retHash = $retXML->toHash();
               if (exists $retHash->{'methodResponse'}->{'fault'}) {
                    $xml_retcode = xmlrpc_get_fault($retHash,'faultCode');
                    $code = lc(xmlrpc_get_fault($retHash,'faultString'));
               } else {
                    $xml_retcode = xmlrpc_get_code($retHash,'code');
                    $code = lc(xmlrpc_get_code($retHash,'msg'));
               }
               if (defined $xml_retcode && $xml_retcode != 1000) {
                    $code = lc(xmlrpc_get_fault($retHash,'faultString'));
               } else {
                    $code = 'ok';
               }
               map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                    $result{$_->{'id'}}{'code'} = $code} @{$uh};
               return ($error,$errortext,%result);
          } else {
               map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                    $result{$_->{'id'}}{'code'} = $errortext} @{$uh};
               return ($error,$errortext,%result);
          }
     } else {
          map {$result{$_->{'id'}}{'host'} = $_->{'host'};
               $result{$_->{'id'}}{'code'} = 'int: Could not build update url'} @{$uh};
          return (1000,'int: Could not build update url',%result);
     }
}

sub xmlrpc_encode_request {
     my $method = shift;
     my $args = shift;
     my $encoding = shift;
     $encoding ||= '';

     my $xmlDoc = XML::Mini::Document->new();
     my $xmlDocRoot = $xmlDoc->getRoot();
     my $xmlHeader = $xmlDocRoot->header('xml');
     $xmlHeader->attribute('encoding', 'UTF-8') if ($encoding);
     $xmlHeader->attribute('version', '1.0');
     my $xmlMethodCall = $xmlDocRoot->createChild('methodCall');
     $xmlMethodCall->createChild('methodName')->text($method);
     my $xmlParams = $xmlMethodCall->createChild('params');
     my $xmlParam = $xmlParams->createChild('param');
     my $xmlValue = $xmlParam->createChild('value');
     my $xmlStruct = $xmlValue->createChild('struct');
     foreach my $a (keys %{$args}) {
          my $xmlMember = $xmlStruct->createChild('member');
          $xmlMember->createChild('name')->text($a);
          my $xmlMemberName = $xmlMember->createChild('value');
          if (looks_like_number($args->{$a})) {
               $xmlMemberName->createChild('int')->text($args->{$a});
          } else {
               $xmlMemberName->createChild('string')->text($args->{$a});
          }
     }
     my $xml = $xmlDoc->toString($XML::Mini::NoWhiteSpaces);
     $xml =~ s/\?\sxml/?xml /;
     $xml =~ s/\"/\\\"/g;
     return $xml;
}
sub xmlrpc_get_code {
     my $xml = shift;
     my $search = shift;

     my @xmlret = grep{$_->{'name'} eq $search} @{$xml->{'methodResponse'}->{'params'}->{'param'}->{'value'}->{'struct'}->{'member'}};
     if (scalar(@xmlret) == 1) {
          my $valuetype = (keys %{$xmlret[0]->{'value'}})[0];
          return $xmlret[0]->{'value'}->{$valuetype};
     }
     return undef;
}
sub xmlrpc_get_fault {
     my $xml = shift;
     my $search = shift;

     my @xmlret = grep{$_->{'name'} eq $search} @{$xml->{'methodResponse'}->{'fault'}->{'value'}->{'struct'}->{'member'}};
     if (scalar(@xmlret) == 1) {
          my $valuetype = (keys %{$xmlret[0]->{'value'}})[0];
          return $xmlret[0]->{'value'}->{$valuetype};
     }
     return undef;
}
sub xmlrpc_get_recordid {
     my $xml = shift;

     # parse till resData
     my @xmlret = grep{$_->{'name'} eq "resData"} @{$xml->{'methodResponse'}->{'params'}->{'param'}->{'value'}->{'struct'}->{'member'}};
     if (scalar(@xmlret) == 1) {
          # parse till record
          my @xmlret = grep{$_->{'name'} eq "record"} @{$xmlret[0]->{'value'}->{'struct'}->{'member'}};
          if (scalar(@xmlret) == 1) {
               my @xmlret = grep{$_->{'name'} eq "id"} @{$xmlret[0]->{'value'}->{'array'}->{'data'}->{'value'}->{'struct'}->{'member'}};
               if (scalar(@xmlret) == 1) {
                    my $valuetype = (keys %{$xmlret[0]->{'value'}})[0];
                    return $xmlret[0]->{'value'}->{$valuetype};
               }
          }
     }
     return undef;
}
1;