#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       Module::kontent                                      #
#  Description: module to update kontent.com ddns hosts              #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2014-2016 by QTip                                    #
#  License:     BSD-style license (see LICENSE)                      #
#  ----------------------------------------------------------------  #
#  Version:     0.41 - 05/05/2016                                    #
#  for more information check the changelog                          #
#********************************************************************#

package DDNSUpdater::Module::kontent;

use strict;
use warnings;

use base qw(Exporter);

our $VERSION     = '0.41';
our @EXPORT      = ();
our @EXPORT_OK   = qw(update_protocol);

use DDNSUpdater::Logger;
use Digest::MD5 qw(md5_hex);
use if $ENV{'DDU_DEBUG'}, "Data::Dumper";

my $EOL = $::EOL;
my $debuglog_anonym = $::debuglog_anonym;

sub update_protocol {
     my $class  = shift;
     my $updatevars  = shift;
     my $uh = shift;

     my ($error,$errortext,$ret,$pw);
     my %result = ();
     $pw = DDNSUpdater::CryptDecrypt->decrypt_data($updatevars->{'tags'}->{$updatevars->{'passtag'}},'','');
     $updatevars->{'tags'}->{$updatevars->{'passtag'}} = md5_hex($pw) if ($pw ne "");
     $pw = '';

     # canonify url
     my $postdata;
     my $updateurl = DDNSUpdater::Tools->canonify_url($updatevars);
     $postdata = ($updatevars->{'req_method'} eq "POST" ? $updatevars->{'tags'} : {});

     # start update process
     if ($updateurl ne "") {
          my $updateurl_ = $updateurl;
          $updateurl_ =~ s/($updatevars->{'logintag'}|$updatevars->{'passtag'})=([^&]*)/$1=xxx/g if ($debuglog_anonym);
          print $updateurl_.$EOL.$EOL if $ENV{'DDU_DEBUG'};
          debug($updateurl_,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($updateurl,$updatevars->{'ipv6'},$updatevars->{'ssl'},$updatevars->{'auth'},$updatevars->{'req_method'},$postdata);
          $ret = DDNSUpdater::Functions->trim(lc($ret));
          debug("return: '".$ret."'",$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          print "return: '".$ret."'".$EOL if $ENV{'DDU_DEBUG'};

          # evaluating response (protocol specific code)
          # return scheme must be $error,$errortext,$codeHash
          # $error:     error number
          # $errortext: error description
          # $codeHash:  the prepared unique return code hash, hash with errortext as code if error != 0
          #             format:
          #                   {
          #                     'host id(0)' => {
          #                        'host' => 'xxx.xxx.xxx.xx',
          #                        'code' => 'return/error-code'
          #                     }
          #                   }

          if ($error == 0) {
               my $code;
               if ($ret =~ /^.*?ok/) {
                    $code = 'ok';
               } else {
                    $code = 'failed';
               }
               map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                    $result{$_->{'id'}}{'code'} = $code} @{$uh};
               return ($error,$errortext,%result);
          } else {
               map {$result{$_->{'id'}}{'host'} = $_->{'host'};
                    $result{$_->{'id'}}{'code'} = $errortext} @{$uh};
               return ($error,$errortext,%result);
          }
     } else {
          map {$result{$_->{'id'}}{'host'} = $_->{'host'};
               $result{$_->{'id'}}{'code'} = 'int: Could not build update url'} @{$uh};
          return (1000,'int: Could not build update url',%result);
     }
}
1;