#********************************************************************#
#  DDNS Updater - 3rdparty dynamic ddns updater                      #
#  Modul:       Tools.pm                                             #
#  Description: misc tools & functions                               #
#  Author:      QTip from the german Synology support forum          #
#  Copyright:   2014-2016 by QTip                                    #
#  License:     BSD-style license (see LICENSE)                      #
#  ----------------------------------------------------------------  #
#  Version:     1.0 - 26/06/2016                                     #
#  for more information check the changelog                          #
#********************************************************************#

package DDNSUpdater::Tools;

use strict;
use warnings;

use base qw(Exporter);

our $VERSION     = '1.0';
our @EXPORT      = qw($ips $useragent $EOL $notifyuser $dsmbuild $notify_enabled $debuglog_anonym);
our @EXPORT_OK   = qw(detect_ip_system analyze_ip api_request get_ip domain2ip canonify_url check_result build_updatevars exec_script);

our $ips = {'ipv4' => undef, 'ipv6' => undef};
our $EOL = "\x0A";
our $useragent;
our $notifyuser = '@administrators';
our $dsmbuild = `/bin/get_key_value /etc.defaults/VERSION buildnumber`;
chomp($dsmbuild);
our $notify_enabled;
our $debuglog_anonym;

use DDNSUpdater::Functions;
use DDNSUpdater::Logger;

use Scalar::Util qw(looks_like_number);
use POSIX qw(strftime);
use Plient qw(plient plient_support);
use Socket;
if ($dsmbuild >= 4458) {
     require Socket6;
     Socket6->import();
}
use Data::Dumper;

sub detect_ip_system {
     my $class = shift;
     my $sites = shift;

     my ($ipv6,$error,$errortext,$ret,$ip,$iptype,$count,$all);

     foreach my $ipt (keys %$ips) {
          print "$ipt".$EOL if ($ENV{'DDU_DEBUG'});
          $ipv6 = ($ipt eq "ipv6") ? 1 : 0;
          $count = 0;
          $all = scalar(grep {$_->{'active'} eq eTRUE && $_->{'ipv6'} eq $ipv6} @{$sites});
          foreach my $weburl (@{$sites}) {
               next if ($weburl->{'active'} != 1);
               next if ($weburl->{'ipv6'} != $ipv6);
               print $weburl->{'url'}.$EOL if ($ENV{'DDU_DEBUG'});
               ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($weburl->{'url'},$ipv6,0);
               print $error." - ".$errortext." - ".$ret.$EOL.$EOL if ($ENV{'DDU_DEBUG'});
               $count++;
               if ($error != 0) {
                    print "DEBUG: using $weburl->{'url'} [$count/$all], error: ".$errortext.$EOL.$EOL if ($ENV{'DDU_DEBUG'});
                    next;
               }
               ($ip,$iptype) = DDNSUpdater::Tools->analyze_ip($ret,$weburl->{'pattern'});
               print $ip." - ".$iptype.$EOL.$EOL if ($ENV{'DDU_DEBUG'});
               if (!defined $ip || $ip eq "" || $iptype ne $ipt) {
                    print "DEBUG: $weburl->{'url'} returned no valid ".$ipt." ip" if ($ENV{'DDU_DEBUG'});
                    print $EOL.$EOL if ($count >= $all && $ENV{'DDU_DEBUG'});
                    print ", trying next server".$EOL.$EOL if ($count < $all && $ENV{'DDU_DEBUG'});
                    warning("$weburl->{'url'} returned no valid ".$ipt." ip".($count < $all?", trying next server":($count >= $all?", no more sites available, terminate task!":"")));
                    next;
               } else {
                    print "DEBUG: using $weburl->{'url'} to get current external ".$ipt." ip".$EOL.$EOL if ($ENV{'DDU_DEBUG'});
                    debug($ip." - ".$iptype,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
                    debug("using $weburl->{'url'} to get current external ".$ipt." ip") if ($ENV{'DDU_DEBUG_LOG'});
                    $ips->{$ipt} = $ip;
                    last;
               }
          }
     }
     return $ips;
}
sub analyze_ip {
     # IPv6 Regex by Dartware, LLC http://intermapper.com
     # Creative Commons Attribution-ShareAlike 3.0 Unported License

     my $class = shift;

     my ($ret,$pattern) = @_;

     my ($ip,$iptype);
     $ip = '';
     $iptype = '';
     $ret =~ s/^.*?$pattern//is;
     if ($ret =~ /^.*?\b(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\b.*/is) {
          $ip = $1;
          $iptype = 'ipv4';
     } elsif ($ret =~ /^.*?\b((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))\b.*/is) {
          $ip = $1;
          $iptype = 'ipv6';
     }
     return ($ip,$iptype);
}
sub api_request {
     my $class = shift;
     my ($url,$ipv6,$ssl,$auth,$rqmethod,$pdata,$referer,$ctime,$mtime) = @_;

     $ctime              ||= 10;
     $mtime              ||= 30;
     $ssl                ||= 0;
     $auth               ||= {'login' => undef, 'password' => undef};
     $rqmethod           ||= 'GET';
     $pdata              ||= {};
     $referer            ||= '';
     $useragent          ||= '';
     my $keep_alive      = 30;
     my $tcp_nodelay     = '';
     my $proto = 'http';
     my $dubody = {
          '--ipv6'            => $ipv6,
          '--connect-timeout' => $ctime,
          '--max-time'        => $mtime,
          '--keepalive-time'  => $keep_alive,
          '--tcp-nodelay'     => $tcp_nodelay
     };
     $dubody->{'--referer'} = $referer if ($referer);
     my $postbody = {%{$dubody}, %{$pdata}};
#print Dumper($auth);
#print Dumper($postbody);
#exit;
     if ($ssl) {
          $url =~ s/http:/https:/i;
          $proto = 'https';
     } else {
          $url =~ s/https:/http:/i;
     }
     my ($ret,$code,$http_code,$curl_code,$err);
     $ENV{'PLIENT_DEBUG'} = $ENV{'DDU_DEBUG'};
     my $webpost = plient_support($proto, $rqmethod);
     if ($webpost) {
          $ret = plient(
               $rqmethod,
               $url,
               {
                    'headers' => {
                         'User-Agent' => $useragent
                    },
                    'user' => $auth->{'login'},
                    'password' => $auth->{'password'},
                    'body' => $postbody
               }
          );
#print $ret;
          # if there is always a return code beside http and curl error, filter http and curl error to get provider response
          # prio #1
          $err = 0;
          $code = $ret;
          $ret =~ s/(http_code\:|curl\:)(.*?)$//g;

          if ($code =~ m/curl:(.*?)\n/) {
          # prio #2 - curl error code
               $curl_code = DDNSUpdater::Functions->trim($1);
               $err = ($curl_code ne "" ? 1000 : 0);
               $code = "curl:".$curl_code;
#print "curl:$curl_code\n";
          } elsif ($code =~ m/http_code\:(.*?)\n/) {
          # prio #3 - http error code
               $http_code = DDNSUpdater::Functions->trim($1);
               $err = ($http_code == 0 || $http_code >= 400 ? 1000 : 0);
               $http_code = ($http_code == 0) ? '000' : $http_code;
               $code = "http:".$http_code;
#print "http:$http_code\n";
          }
#print $err,$code,$ret;
#exit;
          return ($err,$code,$ret);
     } else {
          return (1000,uc($proto)." not available","");
     }
}
sub _ip_intern {
     my @ipsystem = @{(shift)};
     my $sites = shift;

     my ($path,$ipt_,@ipsystem_,$fallback);
     foreach my $ipt (@ipsystem) {
          $fallback = 0;
          $ipt_ = $ipt;
          system('/usr/syno/sbin/ddnsd -e > /dev/null');
          if ($dsmbuild >= 4977) {
               $ipt_ =~ s/ip/IP/;
               $path = '/tmp/ddns.info';
               $ips->{$ipt} = `/bin/get_key_value $path $ipt_`;
               chomp $ips->{$ipt};
          } else {
               if ($ipt eq "ipv6") {
                    # using ipv6 extern method for dsm < 5.1 as fallback
                    @ipsystem_ = ($ipt);
                    $ips = _ip_extern(\@ipsystem_,$sites);
               } else {
                    $path = `/usr/bin/find /tmp /run -name externalIP* 2> /dev/null`;
                    chomp $path;
                    $ips->{$ipt} = `/bin/get_key_value $path externalIP`;
                    chomp $ips->{$ipt};
               }
          }
#info($ips->{$ipt});
          # fallback to external get ip, if dsm returned ip of 0.0.0.0 or 0:0:0:0:0:0:0:0
          if (defined $ips->{$ipt} && (($ipt eq "ipv4" && $ips->{'ipv4'} eq "0.0.0.0") || ($ipt eq "ipv6" && $ips->{'ipv6'} eq "0:0:0:0:0:0:0:0"))) {
               @ipsystem_ = ($ipt);
               print "DEBUG: fallback to external method, because internal method returned no valid ".$ipt." ip (".$ips->{$ipt}.")".$EOL if ($ENV{'DDU_DEBUG'});
               info("fallback to external method, because internal method returned no valid ".$ipt." ip (".$ips->{$ipt}.")");
               debug("fallback to external method, because internal method returned no valid ".$ipt." ip (".$ips->{$ipt}.")");
               $ips = _ip_extern(\@ipsystem_,$sites);
               $fallback = 1 if (defined $ips->{$ipt});
          }
          if ((!defined $ips->{$ipt}) || $ips->{$ipt} eq "" || defined $ips->{$ipt} && (($ipt eq "ipv4" && $ips->{$ipt} eq "0.0.0.0") || ($ipt eq "ipv6" && $ips->{$ipt} eq "0:0:0:0:0:0:0:0"))) {
               print "DEBUG: dsm internal method returned no valid ".$ipt." ip (".$ips->{$ipt}."), terminate task!".$EOL.$EOL if ($ENV{'DDU_DEBUG'});
               error("dsm internal method returned no valid ".$ipt." ip (".$ips->{$ipt}."), terminate task!");
          } else {
               print "DEBUG: using dsm internal method to get current external ".$ipt." ip".$EOL if (!$fallback && $ENV{'DDU_DEBUG'});
               print "DEBUG: ".$ips->{$ipt}." - ".$ipt.$EOL.$EOL if (!$fallback && $ENV{'DDU_DEBUG'});
               debug($ips->{$ipt}." - ".$ipt,$debuglog_anonym) if (!$fallback && $ENV{'DDU_DEBUG_LOG'});
               debug("using dsm internal method to get current external ".$ipt." ip") if (!$fallback && $ENV{'DDU_DEBUG_LOG'});
          }
     }
     return $ips;
}
sub _ip_extern {
     my @ipsystem = @{(shift)};
     my $sites = shift;

     my ($error,$errortext,$ret,$ip,$iptype,$ipv6,$count,$all);
     foreach my $ipt (@ipsystem) {
          $ipv6 = ($ipt eq "ipv6") ? 1 : 0;
          $count = 0;
          $all = scalar(grep {$_->{'active'} eq eTRUE && $_->{'ipv6'} eq $ipv6} @{$sites});
          foreach my $weburl (@{$sites}) {
               next if ($weburl->{'active'} != 1);
               next if ($weburl->{'ipv6'} != $ipv6);
               print "DEBUG: ".$weburl->{'url'}.$EOL.$EOL if ($ENV{'DDU_DEBUG'});
               ($error,$errortext,$ret) = DDNSUpdater::Tools->api_request($weburl->{'url'},$ipv6,0);
               $count++;
               if ($error != 0) {
                    print "using $weburl->{'url'} [$count/$all], error: ".$errortext.$EOL.$EOL if ($ENV{'DDU_DEBUG'});
                    next;
               }
               ($ip,$iptype) = DDNSUpdater::Tools->analyze_ip($ret,$weburl->{'pattern'});
               if (!defined $ip || $ip eq "" || $iptype ne $ipt || defined $ip && (($ipt eq "ipv4" && $ip eq '0.0.0.0') || ($ipt eq "ipv6" && $ip eq '0:0:0:0:0:0:0:0'))) {
                    print "DEBUG: $weburl->{'url'} returned no valid ".$ipt." ip (".$ip.")" if ($ENV{'DDU_DEBUG'});
                    print ", no more sites available, terminate task!".$EOL.$EOL if ($count >= $all && $ENV{'DDU_DEBUG'});
                    print ", trying next server".$EOL.$EOL if ($count < $all && $ENV{'DDU_DEBUG'});
                    warning("$weburl->{'url'} returned no valid ".$ipt." ip (".$ip.")".($count < $all?", trying next server":($count >= $all?", no more sites available, terminate task!":"")));
                    next;
               } else {
                    print "DEBUG: using $weburl->{'url'} to get current ".$ipt." ip".$EOL if ($ENV{'DDU_DEBUG'});
                    print "DEBUG: ".$ip." - ".$iptype.$EOL.$EOL if ($ENV{'DDU_DEBUG'});
                    debug($ip." - ".$iptype,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
                    debug("using $weburl->{'url'} to get current ".$ipt." ip") if ($ENV{'DDU_DEBUG_LOG'});
                    $ips->{$ipt} = $ip;
                    last;
               }
          }
     }
     return $ips;
}
sub get_ip {
     my $class = shift;
     my $ipdetect = shift;
     my @ipsystem = @{(shift)};
     my $sites = shift;

     if ($ipdetect eq "extern") {
          $ips = _ip_extern(\@ipsystem,$sites);
     } else {
          $ips = _ip_intern(\@ipsystem,$sites);
     }
}
sub domain2ip {
     my $class = shift;
     my $domain = shift;
     my $iptype = shift;

     my $ghr;
     my $dom2ip = '';
     if ($iptype eq "syncdom4") {
          $ghr = gethostbyname($domain);
          $dom2ip = inet_ntoa($ghr);
     } elsif ($iptype eq "syncdom6") {
          if ($dsmbuild >= 4458) {
               $ghr = gethostbyname2($domain, AF_INET6);
               $dom2ip = inet_ntop(AF_INET6, $ghr) if ($ghr);
          } else {
               $dom2ip = `/usr/bin/nslookup $domain | /usr/bin/cut -d " " -f3 | /bin/egrep '[0-9a-f]{1,4}:'`;
               chomp $dom2ip;
          }
     }
     return $dom2ip;
}
sub exec_script {
     my $class = shift;
     my $script = shift;
     my $value1 = shift;
     my $value2 = shift;

     $value1 = (defined $value1 && $value1 ne "" ? $value1 : '-');
     $value2 = (defined $value2 && $value2 ne "" ? $value2 : '-');
     if (defined $script) {
          if (-x $script) {
               system("$script $value1 $value2 &");
          } else {
               warning("Could not execute script: $script");
          }
     }
}
sub group_hosts {
     my $class = shift;
     my $hosts = shift;
     my $groupkeys = shift;

#print Dumper($hosts);
#print Dumper($groupkeys);

     my %groups = ();
     foreach my $hc (@{$hosts}) {
          $hc->{'password'} = DDNSUpdater::CryptDecrypt->decrypt_data($hc->{'password'},'','');
          map {$hc->{$_} = '' unless exists($hc->{$_})} @{$groupkeys};
          my $sig = (!scalar(@{$groupkeys}) ? $hc->{'id'} : join('-', map{"$hc->{$_}"} @{$groupkeys}));
          push (@{$groups{$sig}},{'id' => int($hc->{'id'}), 'host' => $hc->{'host'}});
     }
#     print Dumper(\%groups);
     return %groups;
}
sub canonify_url {
     # build canonical uri

     my $class = shift;
     my $updatevars = shift;

     use URI;
     use URI::QueryParam;

     $class->parse_extra_params($updatevars->{'static_params'},$updatevars->{'tags'}) if ($updatevars->{'static_params'} ne "");

     my $url = URI->new($updatevars->{'server'});
     $url->path($updatevars->{'script'});
     $url->query_form_hash($updatevars->{'tags'}) if ($updatevars->{'req_method'} eq "GET");
     $url->canonical;
     return $url;
}
sub parse_extra_params {
     # parse extra params and store in tags hash

     my $class = shift;
     my $query = shift;
     my $tags = shift;

     my (@parts,$part,$value,$name);
     @parts = split(/\&/,$query);
     foreach $part (@parts) {
          ($name,$value) = split(/=/,$part);
          $value =~ tr/+/ /;
          $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
          # prevent include of server side includes
          $value =~ s/<!--(.|\n)*-->//g;
          $tags->{$name} = $value;
     }
}
sub check_result {
     my $class = shift;
     my $error = shift;
     my $errortext = shift;
     my %result = %{(shift)};
     my %rgroup = %{(shift)};
     my $hostconfig = shift;
     my $dustatus = shift;
     my $scale_units = shift;
     my $minerrinterval = shift;

     my ($pdebug,$waittime,$un,$waitsec,$synced);
     my @codefound_ok = ();
     my @codefound_nochg = ();
     my @codefound_tosoon = ();
     print "DEBUG: Check result:".$EOL if ($ENV{'DDU_DEBUG'});
     debug("Check result:") if ($ENV{'DDU_DEBUG_LOG'});
     foreach my $hostid (keys %result) {
#          print $hostid." = ".$result{$hostid}{'code'}.", ".$result{$hostid}{'host'}."\n"; #Dumper($result{$hostid});
          $dustatus->{$hostid}{'status'} = $errortext;
#          $dustatus->{$hostid}{'ip'} = '0';
          $dustatus->{$hostid}{'utime'} = 0;
          $dustatus->{$hostid}{'wtime'} = 0;
          $synced = '';
          if (($hostconfig->{'iptype'} eq "syncdom4" || $hostconfig->{'iptype'} eq "syncdom6") && defined $hostconfig->{'syncdom'}) {
               $synced = ", synced with ".$hostconfig->{'syncdom'};
          }
          if ($error == 0) {
               @codefound_ok = grep {$result{$hostid}{'code'} =~ /\Q$_\E/} @{$rgroup{'OK'}};
               @codefound_nochg = grep {$result{$hostid}{'code'} =~ /\Q$_\E/} @{$rgroup{'NOCHG'}};
               @codefound_tosoon = grep {$result{$hostid}{'code'} =~ /\Q$_\E/} @{$rgroup{'TOSOON'}};
#print "--ok--     : ".Dumper(@codefound_ok)."\n";
#print "--nochg--  : ".Dumper(@codefound_nochg)."\n";
#print "--tosoon-- : ".Dumper(@codefound_tosoon)."\n";
               if (scalar @codefound_ok) {
                    $dustatus->{$hostid}{'status'} = $codefound_ok[0];
                    $dustatus->{$hostid}{'ip'} = $hostconfig->{'ip'};
                    $dustatus->{$hostid}{'ltime'} = time();
                    success("updating ".$result{$hostid}{'host'}.", status: ".$dustatus->{$hostid}{'status'}.", IP set to ".$dustatus->{$hostid}{'ip'}.$synced);
               } elsif (scalar @codefound_nochg) {
                    $dustatus->{$hostid}{'status'} = $codefound_nochg[0];
                    $dustatus->{$hostid}{'ip'} = $hostconfig->{'ip'};
                    $dustatus->{$hostid}{'ltime'} = time();
                    warning("updating ".$result{$hostid}{'host'}.", status: ".$dustatus->{$hostid}{'status'}.", IP set to ".$dustatus->{$hostid}{'ip'}.$synced.", No update required; unnecessary attempts to change to the current address are considered abusive");
               } elsif (scalar @codefound_tosoon) {
                    ($waittime, $un) = (($minerrinterval/60), 'm');
                    $dustatus->{$hostid}{'status'} = $codefound_tosoon[0];
                    $dustatus->{$hostid}{'utime'} = (time() + $minerrinterval);
                    warning("updating ".$result{$hostid}{'host'}.", status: ".$dustatus->{$hostid}{'status'}.", Please wait $waittime $un before perform next update!");
               } elsif ($result{$hostid}{'code'} =~ m/w(\d+)(.)/) {
                    ($waittime, $un) = ($1, lc($2));
                    $waitsec = ($waittime * $scale_units->{$un});
                    $dustatus->{$hostid}{'status'} = $result{$hostid}{'code'};
                    $dustatus->{$hostid}{'wtime'} = (time() + $waitsec);
                    warning("updating ".$result{$hostid}{'host'}.", status: ".$dustatus->{$hostid}{'status'}.", Please wait $waittime $un before perform next update!");
               } else {
                    $dustatus->{$hostid}{'status'} = $result{$hostid}{'code'};
                    $dustatus->{$hostid}{'utime'} = (time() + $minerrinterval);
                    failed("updating ".$result{$hostid}{'host'}.", status: ".$dustatus->{$hostid}{'status'});
                    DDNSUpdater::Functions->sendnotify($notifyuser,"Update of ".$result{$hostid}{'host'}." failed, status: ".$dustatus->{$hostid}{'status'},$dsmbuild) if ($notify_enabled);
               }
               $pdebug = $result{$hostid}{'host'}.": status found: ".$dustatus->{$hostid}{'status'}.", ip set: ".$dustatus->{$hostid}{'ip'}.$synced
                        .", last successful update: ".($dustatus->{$hostid}{'ltime'}?strftime("%Y-%m-%d %H:%M:%S", localtime($dustatus->{$hostid}{'ltime'})):'never')
                        .", wait time: ".($dustatus->{$hostid}{'wtime'}?strftime("%Y-%m-%d %H:%M:%S", localtime($dustatus->{$hostid}{'wtime'})):0)."s"
                        .", ".($dustatus->{$hostid}{'utime'}?"next update attempt: ".strftime("%Y-%m-%d %H:%M:%S", localtime($dustatus->{$hostid}{'utime'})):"");
               print "DEBUG: --------------".$EOL if ($ENV{'DDU_DEBUG'});
               print "DEBUG: ".$pdebug.$EOL.$EOL if ($ENV{'DDU_DEBUG'});
               debug("--------------") if ($ENV{'DDU_DEBUG_LOG'});
               debug($pdebug,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          } else {
               $dustatus->{$hostid}{'utime'} = (time() + $minerrinterval);
               my $last_ = $dustatus->{$hostid}{'status'};
#               my $last_ = (index($result{$hostid}{'code'},'http:') != -1 ? substr($result{$hostid}{'code'},5) : $result{$hostid}{'code'});
               failed("updating ".$result{$hostid}{'host'}.", status: ".$last_);
               DDNSUpdater::Functions->sendnotify($notifyuser,"Update of ".$result{$hostid}{'host'}." failed, status: ".$last_,$dsmbuild) if ($notify_enabled);
               $pdebug = $result{$hostid}{'host'}.": status found: ".$last_
                        .", time: ".($dustatus->{$hostid}{'ltime'}?strftime("%Y-%m-%d %H:%M:%S", localtime($dustatus->{$hostid}{'ltime'})):0)
                        .", wait time: ".($dustatus->{$hostid}{'wtime'}?strftime("%Y-%m-%d %H:%M:%S", localtime($dustatus->{$hostid}{'wtime'})):0)."s"
                        .", ".($dustatus->{$hostid}{'utime'}?"next update attempt: ".strftime("%Y-%m-%d %H:%M:%S", localtime($dustatus->{$hostid}{'utime'})):"");
               print "DEBUG: --------------".$EOL if ($ENV{'DDU_DEBUG'});
               print "DEBUG: ".$pdebug.$EOL.$EOL if ($ENV{'DDU_DEBUG'});
               debug("--------------") if ($ENV{'DDU_DEBUG_LOG'});
               debug($pdebug,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
          }
     }
}
sub build_updatevars {
     # create update object from config hashes

     my $class = shift;
     my $hostconfig = shift;
     my $updatevars = shift;
     my $provider = shift;
     my @protocol = @{(shift)};

     $updatevars->{'ssl'} = $hostconfig->{'ssl'};
     $updatevars->{'ipv6'} = 'auto'; #$hostconfig->{'ipv6'};
     $updatevars->{'ips'} = $ips;

     $updatevars->{'static_params'} = $protocol[0]->{'static_params'};
     $updatevars->{'server'} = ($hostconfig->{'ipv6'} eq eTRUE && exists $provider->{'serveripv6'} && $provider->{'serveripv6'} ne "" && $updatevars->{'ipsystem'} eq "ipv6" ? $provider->{'serveripv6'} : $provider->{'server'});
     $updatevars->{'server'} = ($hostconfig->{'ssl'} eq eTRUE) ? "https://$updatevars->{'server'}" : "http://$updatevars->{'server'}";
     $updatevars->{'script'} = $provider->{'script'};
     $updatevars->{'req_method'} = $provider->{'req_method'};

     # build offline tag
     if ($provider->{'offline_method'} eq "offline1") {
          $updatevars->{'tags'}->{'offline'} = $hostconfig->{'offline'} ? '1' : '0';
     } elsif ($provider->{'offline_method'} eq "offlineon") {
          $updatevars->{'tags'}->{'offline'} = $hostconfig->{'offline'} ? 'ON' : 'OFF';
     } elsif ($provider->{'offline_method'} eq "offliney") {
          $updatevars->{'tags'}->{'offline'} = $hostconfig->{'offline'} ? 'y' : 'n';
     } elsif ($provider->{'offline_method'} eq "offlineyes") {
          $updatevars->{'tags'}->{'offline'} = $hostconfig->{'offline'} ? 'YES' : 'NO';
     } elsif ($provider->{'offline_method'} eq "myipoffline") {
          $hostconfig->{'ip'} = $hostconfig->{'offline'} ? 'offline' : $hostconfig->{'ip'};
     } elsif ($provider->{'offline_method'} eq "ip0") {
          $hostconfig->{'ip'} = $hostconfig->{'offline'} ? '0.0.0.0' : $hostconfig->{'ip'};
     } elsif ($provider->{'offline_method'} eq "customip") {
          $hostconfig->{'ip'} = $hostconfig->{'offline'} ? $updatevars->{'offline_ip'} : $hostconfig->{'ip'};
     }

     # build mxfield and auth tags
     my $mxfields = 0;
     $updatevars->{'logintag'} = ' ';
     $updatevars->{'passtag'} = ' ';
     while (my($tag,$value) = each(%{$protocol[0]{'tags'}[0]})) {
          # check if mx host exists
          $mxfields = 1 if ($value eq "mx" && defined $hostconfig->{$value} && $hostconfig->{$value} ne "");
          $updatevars->{'iptag'} = $tag if ($value eq "ip");
          $updatevars->{'logintag'} = $tag if ($value eq "login");
          $updatevars->{'passtag'} = $tag if ($value eq "password");
          if ($value eq "wildcard" && defined $hostconfig->{$value} && $hostconfig->{$value} ne "") {
               # convert to on/off
               $updatevars->{'tags'}->{$value} = ($hostconfig->{$value} eq eTRUE) ? 'ON' : 'OFF';
          } elsif (($value eq "backmx" || $value eq "static" || $value eq "useroption1") && defined $hostconfig->{$value} && $hostconfig->{$value} ne "") {
               # convert to yes/no
               $updatevars->{'tags'}->{$value} = ($hostconfig->{$value} eq eTRUE) ? 'YES' : 'NO';
          } elsif ($value eq "login" || $value eq "password") {
               # check if auth 'basic' wanted, change hash content to
               # $updatevars->{'auth'} = {};
                if ($provider->{'auth_method'} eq "basic") {
#print $tag." - ".$value."\n";
                    $updatevars->{'auth'}->{$value} = $hostconfig->{$value};
               } else {
                    $updatevars->{'tags'}->{$tag} = $hostconfig->{$value} if (defined $hostconfig->{$value} && $hostconfig->{$value} ne "");
               }
          } else {
               $updatevars->{'tags'}->{$tag} = $hostconfig->{$value} if (defined $hostconfig->{$value} && $hostconfig->{$value} ne "");
          }
     }
     # switch iptag to ipv6tag, if ipv6tag
     if ($hostconfig->{'ipv6'} eq eTRUE && exists $updatevars->{'ipv6tag'} && $updatevars->{'ipv6tag'} ne "" && exists $updatevars->{'tags'}->{$updatevars->{'iptag'}}) {
          $updatevars->{'tags'}->{$updatevars->{'ipv6tag'}} = $updatevars->{'tags'}->{$updatevars->{'iptag'}};
          delete $updatevars->{'tags'}->{$updatevars->{'iptag'}};
          $updatevars->{'iptag'} = $updatevars->{'ipv6tag'};
     }

     print $EOL."DEBUG: Build updatevars:".$EOL if ($ENV{'DDU_DEBUG'});
     debug("Build updatevars:") if ($ENV{'DDU_DEBUG_LOG'});
     print "DEBUG: protocol: ".$provider->{'protocol'}.$EOL if ($ENV{'DDU_DEBUG'});
     debug("protocol: ".$provider->{'protocol'}) if ($ENV{'DDU_DEBUG_LOG'});
     print "DEBUG: module: ".$protocol[0]->{'module'}.$EOL if ($ENV{'DDU_DEBUG'});
     debug("module: ".$protocol[0]->{'module'}) if ($ENV{'DDU_DEBUG_LOG'});
     print "DEBUG: mxfields: $mxfields".$EOL if ($ENV{'DDU_DEBUG'});
     debug("mxfields: $mxfields",$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
     # delete all mxfields, if mx host empty
     if ($mxfields == 0) {
          my @unwanted = grep {/.*mx.*/} keys %{$updatevars->{'tags'}};
          delete @{$updatevars->{'tags'}}{@unwanted};
     }
     print "DEBUG: authmethod: ".$provider->{'auth_method'}.$EOL if ($ENV{'DDU_DEBUG'});
     debug("authmethod: ".$provider->{'auth_method'}) if ($ENV{'DDU_DEBUG_LOG'});
     print "DEBUG: ".Dumper($updatevars) if ($ENV{'DDU_DEBUG'});
     debug($updatevars,$debuglog_anonym) if ($ENV{'DDU_DEBUG_LOG'});
     print "DEBUG: --------------".$EOL if ($ENV{'DDU_DEBUG'});
     debug("--------------") if ($ENV{'DDU_DEBUG_LOG'});
}
1;